@@ -3,6 +3,7 @@ use std::{collections::BTreeSet, hash::Hash, iter::zip};
 use num_bigint::BigUint;
 use pyo3::prelude::*;
 use rr_util::{
+    name::Name,
     opt_einsum::EinsumSpec,
     tensor_util::{TensorAxisIndex, TensorIndex, TorchDeviceDtype, TorchDtype},
 };
@@ -32,7 +33,7 @@ pub fn cast_circuit(
                 {
                     Some(Array::nrc(
                         device_dtype.cast_tensor(node.value.clone()),
-                        node.name_cloned(),
+                        node.info().name,
                     ))
                 } else {
                     None
@@ -71,7 +72,7 @@ pub fn cast_circuit(
                                 })
                                 .collect(),
                         ),
-                        node.name_cloned(),
+                        node.info().name,
                     ))
                 } else {
                     None
@@ -319,7 +320,7 @@ pub fn replace_all_randn_seeded(circuit: CircuitRc) -> CircuitRc {
     deep_map_unwrap(circuit, |node| match &**node {
         Circuit::Array(ac) => Array::randn_full(
             ac.info().shape.clone(),
-            ac.name_cloned(),
+            ac.info().name,
             ac.info().device_dtype.clone(),
             // We use the tensor hash instead of the circuit hash
             Some(ac.value.hash_usize().unwrap()),
@@ -342,7 +343,7 @@ pub fn replace_all_randn_seeded(circuit: CircuitRc) -> CircuitRc {
                     })
                     .collect(),
             ),
-            index.name_cloned(),
+            index.info().name,
         ),
         _ => node.clone(),
     })
@@ -369,8 +370,10 @@ pub fn replace_nodes(circuit: CircuitRc, map: &HashMap<HashBytes, CircuitRc>) ->
 #[pyfunction]
 pub fn prefix_all_names(circuit: CircuitRc, prefix: String) -> CircuitRc {
     deep_map_unwrap(circuit, |x| {
-        if let Some(name) = x.name() {
-            return x.clone().rename(Some(prefix.clone() + name));
+        if let Some(name) = x.info().name {
+            return x
+                .clone()
+                .rename(Some((prefix.clone() + name.into()).into()));
         }
         x
     })
@@ -432,10 +435,10 @@ pub fn do_add_parenthesis_to_name(parent_priority: &OperatorPriority, child: Cir
 pub fn child_name_with_maybe_paren(
     parent_priority: &OperatorPriority,
     child: CircuitRc,
-) -> Option<String> {
-    child.name_cloned().map(|name| {
+) -> Option<Name> {
+    child.info().name.map(|name| {
         if do_add_parenthesis_to_name(parent_priority, child) {
-            format!("({name})")
+            format!("({name})").into()
         } else {
             name
         }
@@ -446,15 +449,15 @@ pub fn child_name_with_maybe_paren(
 pub fn children_names_with_maybe_paren(
     parent_priority: &OperatorPriority,
     children: Vec<CircuitRc>,
-) -> Option<Vec<String>> {
-    if children.iter().any(|x| x.name().is_none()) {
+) -> Option<Vec<Name>> {
+    if children.iter().any(|x| x.info().name.is_none()) {
         None
     } else {
         Some(
             children
                 .into_iter()
                 .map(|x| child_name_with_maybe_paren(parent_priority, x).unwrap())
-                .collect::<Vec<String>>(),
+                .collect::<Vec<Name>>(),
         )
     }
 }
