@@ -43,7 +43,8 @@ use crate::{
     },
     new_rc_unwrap,
     prelude::*,
-    CachedCircuitInfo, CircuitNodeSelfOnlyHash, ConstructError, PyCircuitBase, Scalar, Shape,
+    upcast_tensors_for_eval, CachedCircuitInfo, CircuitNodeSelfOnlyHash, ConstructError,
+    PyCircuitBase, Scalar, Shape,
 };
 
 /// our Einsum supports diag output
@@ -130,10 +131,11 @@ impl CircuitNode for Einsum {
         self.get_spec().flops()
     }
 
-    fn eval_tensors(&self, tensors: &[Tensor], device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
+    fn eval_tensors(&self, tensors: &[Tensor]) -> Result<Tensor> {
+        let tensors = upcast_tensors_for_eval!(self, tensors);
         Python::with_gil(|py| {
             if self.args.is_empty() {
-                return Ok(scalar_to_tensor_py(py, 1., sv![], device_dtype.clone())?);
+                return Ok(scalar_to_tensor_py(py, 1., sv![], Default::default())?);
             }
             let out_axes_deduped: EinsumAxes = self.out_axes.iter().unique().cloned().collect();
             let result_non_diag: Tensor = einsum_py(
@@ -693,10 +695,11 @@ impl CircuitNode for Add {
         self.info.numel() * self.nodes.len()
     }
 
-    fn eval_tensors(&self, tensors: &[Tensor], device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
+    fn eval_tensors(&self, tensors: &[Tensor]) -> Result<Tensor> {
         if tensors.is_empty() {
-            return Ok(scalar_to_tensor(0.0, sv!(), device_dtype.clone())?);
+            return Ok(scalar_to_tensor(0.0, sv!(), Default::default())?);
         }
+        let tensors = upcast_tensors_for_eval!(self, tensors);
         let mut out = tensors[0].clone();
         Python::with_gil(|py| {
             for tensor in tensors[1..].iter() {
@@ -834,7 +837,7 @@ impl CircuitNode for Rearrange {
         )
     }
 
-    fn eval_tensors(&self, tensors: &[Tensor], _device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
+    fn eval_tensors(&self, tensors: &[Tensor]) -> Result<Tensor> {
         // avoid using self.spec.apply so we get shape errors correctly
         let (string, letter_sizes) = self
             .conform_to_input_shape_spec()
@@ -1021,8 +1024,9 @@ impl CircuitNode for Index {
         )
     }
 
-    fn eval_tensors(&self, tensors: &[Tensor], _device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
+    fn eval_tensors(&self, tensors: &[Tensor]) -> Result<Tensor> {
         assert_eq!(tensors.len(), 1);
+        let tensors = upcast_tensors_for_eval!(self, tensors);
         Python::with_gil(|py| {
             PY_CIRCUIT_ITEMS
                 .circ_compiler_util
@@ -1345,7 +1349,8 @@ impl CircuitNode for Concat {
         ]
     }
 
-    fn eval_tensors(&self, tensors: &[Tensor], _device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
+    fn eval_tensors(&self, tensors: &[Tensor]) -> Result<Tensor> {
+        let tensors = upcast_tensors_for_eval!(self, tensors);
         Python::with_gil(|py| {
             PY_UTILS
                 .torch
@@ -1526,7 +1531,7 @@ impl CircuitNode for Scatter {
         )
     }
 
-    fn eval_tensors(&self, tensors: &[Tensor], _device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
+    fn eval_tensors(&self, tensors: &[Tensor]) -> Result<Tensor> {
         Python::with_gil(|py| {
             PY_CIRCUIT_ITEMS
                 .circ_compiler_util
@@ -1799,7 +1804,8 @@ impl CircuitNode for Conv {
         )
     }
 
-    fn eval_tensors(&self, tensors: &[Tensor], _device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
+    fn eval_tensors(&self, tensors: &[Tensor]) -> Result<Tensor> {
+        let tensors = upcast_tensors_for_eval!(self, tensors);
         pycall!(
             PY_UTILS.conv,
             (
