@@ -76,11 +76,7 @@ impl CircuitNodeHashItems for Array {
 impl CircuitNode for Array {
     circuit_node_auto_leaf_impl!("b2aac9d5-1bfa-4c2a-9684-e3f9ecbc1b94");
 
-    fn eval_tensors(
-        &self,
-        _tensors: &[Tensor],
-        _device_dtype: &TorchDeviceDtype,
-    ) -> Result<Tensor> {
+    fn eval_tensors(&self, _tensors: &[Tensor]) -> Result<Tensor> {
         Ok(self.value.clone())
     }
 }
@@ -116,7 +112,7 @@ impl Array {
     }
 
     pub fn randn(shape: Shape) -> Self {
-        Self::randn_full(shape, None, Default::default(), None)
+        Self::randn_full(shape, None, TorchDeviceDtypeOp::TENSOR_DEFAULT, None)
     }
 
     pub fn randn_named(shape: Shape, name: Option<Name>, device_dtype: TorchDeviceDtypeOp) -> Self {
@@ -137,7 +133,7 @@ impl Array {
     #[args(
         shape = "*",
         name = "None",
-        device_dtype = "Default::default()",
+        device_dtype = "TorchDeviceDtypeOp::TENSOR_DEFAULT",
         seed = "None"
     )]
     pub fn randn_full(
@@ -238,11 +234,7 @@ impl CircuitNodeHashItems for Symbol {
 impl CircuitNode for Symbol {
     circuit_node_auto_leaf_impl!("13c3ee63-76e9-4afb-8057-40309d17b458");
 
-    fn eval_tensors(
-        &self,
-        _tensors: &[Tensor],
-        _device_dtype: &TorchDeviceDtype,
-    ) -> Result<Tensor> {
+    fn eval_tensors(&self, _tensors: &[Tensor]) -> Result<Tensor> {
         Err(TensorEvalError::NotExplicitlyComputableInternal {
             circuit: self.crc(),
         }
@@ -314,11 +306,11 @@ impl CircuitNodeHashItems for Scalar {
 impl CircuitNode for Scalar {
     circuit_node_auto_leaf_impl!("78a77905-8b3f-4471-bb77-255673941fef");
 
-    fn eval_tensors(&self, _tensors: &[Tensor], device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
+    fn eval_tensors(&self, _tensors: &[Tensor]) -> Result<Tensor> {
         Ok(scalar_to_tensor(
             self.value,
             self.info().shape.clone(),
-            device_dtype.clone(),
+            Default::default(),
         )?)
     }
 }
