@@ -16,7 +16,7 @@ use rr_util::{
     py_types::{assert_tensors_close, ExtraPySelfOps, PyShape, Tensor, PY_UTILS},
     pycall, python_error_exception, simple_from,
     tensor_util::{
-        broadcast_shapes_impl, check_canon_idxs, MiscInputError, Shape, TorchDeviceDtype,
+        broadcast_shapes_impl, check_canon_idxs, MiscInputError, Shape, TorchDevice,
         TorchDeviceDtypeOp, TorchDtype,
     },
 };
@@ -71,6 +71,7 @@ gf_gen!(
     (tanh, 0),
     (rsqrt, 0),
     (gelu, 0),
+    (gelu_new, 0),
     (relu, 0),
     (step, 0),
     (reciprocal, 0),
@@ -115,7 +116,7 @@ pub const OFFICIAL_GENERALFUNCTION_INVERSES: [(&str, &str); 1] = [("reciprocal",
 /// the number of non batchable dims in output, starting from end, is num_non_batchable_output_dims.
 pub trait SpecTrait: fmt::Debug + ToPyObject {
     fn compute_hash(&self) -> HashBytes;
-    fn function(&self, tensors: &[Tensor], device_dtype: &TorchDeviceDtype) -> Result<Tensor>;
+    fn function(&self, tensors: &[Tensor]) -> Result<Tensor>;
     fn get_shape_info(&self, shapes: &[Shape]) -> Result<GeneralFunctionShapeInfo>;
     // fn has_jacobian(&self) -> Result<bool>;
     // fn get_jacobians(&self, func: &GeneralFunction) -> Result<Option<Vec<Circuit>>>;
@@ -217,7 +218,7 @@ impl SpecTrait for GeneralFunctionSimpleSpec {
         *hasher.finalize().as_bytes()
     }
 
-    fn function(&self, tensors: &[Tensor], _device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
+    fn function(&self, tensors: &[Tensor]) -> Result<Tensor> {
         Python::with_gil(|py| {
             Ok(self
                 .get_function()
@@ -276,9 +277,11 @@ impl SpecTrait for GeneralFunctionSimpleSpec {
 #[derive(Debug, Clone)]
 pub struct GeneralFunctionIndexSpec {
     #[pyo3(get)]
-    index_dim: i64,
+    pub index_dim: i64,
     #[pyo3(get)]
-    batch_x: bool,
+    pub batch_x: bool,
+    #[pyo3(get)]
+    pub batch_index: bool,
     #[pyo3(get)]
     check_index_ints: bool,
 }
@@ -293,16 +296,35 @@ impl ToPyObject for GeneralFunctionIndexSpec {
     }
 }
 
+fn index_get_device_dtype_override(
+    device_dtypes: &[TorchDeviceDtypeOp],
+) -> Option<Result<TorchDeviceDtypeOp>> {
+    if let Some(dev1) = device_dtypes[0].device && let Some(dev2) = device_dtypes[1].device && dev1!=dev2{
+            return Some(Err(MiscInputError::ChildrenMultipleDevices { a: device_dtypes[0].device, b: device_dtypes[1].device}.into()))
+        }
+    if let Some(dtype) = device_dtypes[1].dtype && dtype != TorchDtype::int64 {
+            return Some(Err(MiscInputError::IndexDtypeNotI64 {}.into()));
+        }
+    Some(Ok(TorchDeviceDtypeOp {
+        device: device_dtypes[0].device.or(device_dtypes[1].device),
+        dtype: device_dtypes[0].dtype,
+    }))
+}
+
 impl SpecTrait for GeneralFunctionIndexSpec {
     fn compute_hash(&self) -> HashBytes {
         let mut hasher = blake3::Hasher::new();
         hasher.update(uuid!("442fde55-a1b7-4a69-98ff-bd40d7030ef2").as_bytes()); // uuid for Index
         hasher.update(&self.index_dim.to_le_bytes());
-        hasher.update(&[self.batch_x as u8, self.check_index_ints as u8]);
+        hasher.update(&[
+            self.batch_x as u8,
+            self.batch_index as u8,
+            self.check_index_ints as u8,
+        ]);
         *hasher.finalize().as_bytes()
     }
 
-    fn function(&self, tensors: &[Tensor], _device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
+    fn function(&self, tensors: &[Tensor]) -> Result<Tensor> {
         Python::with_gil(|py| {
             Ok(PY_UTILS
                 .gen_index_function
@@ -316,6 +338,7 @@ impl SpecTrait for GeneralFunctionIndexSpec {
                             .chain([
                                 self.index_dim.into_py(py),
                                 self.batch_x.into_py(py),
+                                self.batch_index.into_py(py),
                                 self.check_index_ints.into_py(py),
                             ])
                             .collect::<Vec<_>>(),
@@ -345,7 +368,7 @@ impl SpecTrait for GeneralFunctionIndexSpec {
             batch_x: self.batch_x,
         };
 
-        if self.batch_x {
+        if self.batch_x && self.batch_index {
             let prefix_len = index_shape.len();
             if prefix_len >= x_shape.len() {
                 // condition to ensure that suffix len >= 1
@@ -371,7 +394,7 @@ impl SpecTrait for GeneralFunctionIndexSpec {
                 num_non_batchable_output_dims: (suffix_len - 1) as u8, // sub 1 for indexed
                 input_batchability: vec![true, true].into_iter().collect(),
             })
-        } else {
+        } else if self.batch_index {
             let index_dim = check_canon_idxs(x_shape.len(), &[self.index_dim])
                 .context("index dim out of bounds for x_shape")?[0];
 
@@ -385,6 +408,30 @@ impl SpecTrait for GeneralFunctionIndexSpec {
                 num_non_batchable_output_dims: (x_shape.len() - 1) as u8, // sub 1 for indexed
                 input_batchability: vec![false, true].into_iter().collect(),
             })
+        } else {
+            if self.batch_x && self.index_dim >= 0 {
+                bail!("index dim must be negative if you're only batching over x");
+            }
+            let index_dim = check_canon_idxs(x_shape.len(), &[self.index_dim])
+                .context("index dim out of bounds for x_shape")?[0];
+
+            let shape: Shape = x_shape[..index_dim]
+                .iter()
+                .chain(index_shape)
+                .chain(&x_shape[index_dim + 1..])
+                .cloned()
+                .collect();
+            let lenshape = shape.len();
+
+            Ok(GeneralFunctionShapeInfo {
+                shape,
+                num_non_batchable_output_dims: if self.batch_x {
+                    index_shape.len() + x_shape[index_dim + 1..].len()
+                } else {
+                    lenshape
+                } as u8,
+                input_batchability: vec![self.batch_x, false].into_iter().collect(),
+            })
         }
     }
 
@@ -396,19 +443,179 @@ impl SpecTrait for GeneralFunctionIndexSpec {
         &self,
         device_dtypes: &[TorchDeviceDtypeOp],
     ) -> Option<Result<TorchDeviceDtypeOp>> {
-        if let Some(dev1) = device_dtypes[0].device && let Some(dev2) = device_dtypes[1].device && dev1!=dev2{
-            return Some(Err(MiscInputError::ChildrenMultipleDevices { a: device_dtypes[0].device, b: device_dtypes[1].device}.into()))
-        }
-        if let Some(dtype) = device_dtypes[1].dtype && dtype != TorchDtype::int64 {
-            return Some(Err(MiscInputError::IndexDtypeNotI64 {}.into()));
-        }
-        Some(Ok(device_dtypes[0]))
+        index_get_device_dtype_override(device_dtypes)
     }
     fn name(&self) -> &'static str {
         Name::new(&format!(
-            "gen_index_at_{}{}{}",
+            "gen_index_at_{}{}{}{}",
             self.index_dim,
             if self.batch_x { "_batch_x" } else { "" },
+            if !self.batch_index {
+                "_no_batch_index"
+            } else {
+                ""
+            },
+            if self.check_index_ints { "_c" } else { "" }
+        ))
+        .into()
+    }
+    fn is_official(&self) -> bool {
+        true
+    }
+    fn serialize(&self) -> Result<Option<String>> {
+        Ok(Some(self.name().into()))
+    }
+}
+
+#[pyclass]
+#[derive(Debug, Clone)]
+pub struct GeneralFunctionExplicitIndexSpec {
+    #[pyo3(get)]
+    index_dim: i64,
+    #[pyo3(get)]
+    x_non_batch_dims: usize,
+    #[pyo3(get)]
+    check_index_ints: bool,
+}
+
+simple_from!(
+    |x: GeneralFunctionExplicitIndexSpec| -> GeneralFunctionSpec {
+        GeneralFunctionSpec::ExplicitIndex(x)
+    }
+);
+
+impl ToPyObject for GeneralFunctionExplicitIndexSpec {
+    fn to_object(&self, py: Python<'_>) -> PyObject {
+        self.clone().into_py(py)
+    }
+}
+
+impl GeneralFunctionExplicitIndexSpec {
+    fn canon_index_dim(&self) -> Result<usize> {
+        check_canon_idxs(self.x_non_batch_dims, &[self.index_dim])
+            .with_context(|| {
+                format!(
+                    "index_dim={} out of bounds for x_non_batch_dims={}",
+                    self.index_dim, self.x_non_batch_dims
+                )
+            })
+            .map(|x| x[0])
+    }
+
+    pub fn new(index_dim: i64, x_non_batch_dims: usize, check_index_ints: bool) -> Result<Self> {
+        let out = Self {
+            index_dim,
+            x_non_batch_dims,
+            check_index_ints,
+        };
+
+        out.canon_index_dim()?; // check error
+
+        Ok(out)
+    }
+}
+
+impl SpecTrait for GeneralFunctionExplicitIndexSpec {
+    fn compute_hash(&self) -> HashBytes {
+        let mut hasher = blake3::Hasher::new();
+        hasher.update(uuid!("442fde55-a1b7-4a69-98ff-bd40d7030ef2").as_bytes()); // uuid for Index
+        hasher.update(&self.index_dim.to_le_bytes());
+        hasher.update(&self.x_non_batch_dims.to_le_bytes());
+        hasher.update(&[self.check_index_ints as u8]);
+        *hasher.finalize().as_bytes()
+    }
+
+    fn function(&self, tensors: &[Tensor]) -> Result<Tensor> {
+        Python::with_gil(|py| {
+            Ok(PY_UTILS
+                .explicit_gen_index_function
+                .call(
+                    py,
+                    PyTuple::new(
+                        py,
+                        tensors
+                            .iter()
+                            .map(|x| x.clone().into_py(py))
+                            .chain([
+                                self.index_dim.into_py(py),
+                                self.x_non_batch_dims.into_py(py),
+                                self.check_index_ints.into_py(py),
+                            ])
+                            .collect::<Vec<_>>(),
+                    ),
+                    None,
+                )
+                .context("evaluate explicit gen index")?
+                .extract(py)
+                .unwrap())
+        })
+    }
+
+    fn get_shape_info(&self, shapes: &[Shape]) -> Result<GeneralFunctionShapeInfo> {
+        let (x_shape, index_shape) = if let [x_shape, index_shape] = shapes {
+            (x_shape, index_shape)
+        } else {
+            bail!(GeneralFunctionShapeError::WrongNumShapes {
+                got: shapes.len(),
+                expected: 2
+            });
+        };
+
+        // TODO: improve errors as needed
+        let get_err = || GeneralFunctionShapeError::ExplicitIndexShapeInvalid {
+            x_shape: x_shape.clone(),
+            index_shape: index_shape.clone(),
+            index_dim: self.index_dim,
+            x_non_batch_dims: self.x_non_batch_dims,
+        };
+
+        assert!(self.x_non_batch_dims >= 1);
+        if x_shape.len() < self.x_non_batch_dims {
+            // condition to ensure that x suffix len >= 1
+            bail!(get_err())
+        }
+
+        let batch_len = x_shape.len() - self.x_non_batch_dims;
+
+        if index_shape.len() < batch_len {
+            bail!(get_err())
+        }
+
+        if x_shape[..batch_len] != index_shape[..batch_len] {
+            bail!(get_err())
+        }
+
+        let final_index_dim = self.canon_index_dim().unwrap() + batch_len;
+
+        let shape: Shape = index_shape
+            .iter()
+            .chain(&x_shape[batch_len..final_index_dim])
+            .chain(&x_shape[final_index_dim + 1..])
+            .cloned()
+            .collect();
+        let non_batch = shape.len() - batch_len;
+        Ok(GeneralFunctionShapeInfo {
+            shape,
+            num_non_batchable_output_dims: non_batch as u8,
+            input_batchability: vec![true, true].into_iter().collect(),
+        })
+    }
+
+    // fn has_jacobian(&self) -> Result<bool> {
+    //     Ok(self.get_jacobians.is_some())
+    // }
+    // fn get_jacobians(&self, func: &GeneralFunction) -> Result<Option<Vec<Circuit>>> {}
+    fn get_device_dtype_override(
+        &self,
+        device_dtypes: &[TorchDeviceDtypeOp],
+    ) -> Option<Result<TorchDeviceDtypeOp>> {
+        index_get_device_dtype_override(device_dtypes)
+    }
+    fn name(&self) -> &'static str {
+        Name::new(&format!(
+            "explicit_index_at_{}_x_non_b_{}{}",
+            self.index_dim,
+            self.x_non_batch_dims,
             if self.check_index_ints { "_c" } else { "" }
         ))
         .into()
@@ -448,7 +655,7 @@ impl SpecTrait for GeneralFunctionSetDDSpec {
         *hasher.finalize().as_bytes()
     }
 
-    fn function(&self, tensors: &[Tensor], _device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
+    fn function(&self, tensors: &[Tensor]) -> Result<Tensor> {
         pycall!(
             PY_UTILS.cast_tensor,
             (tensors[0].clone(), self.output),
@@ -556,7 +763,7 @@ impl SpecTrait for GeneralFunctionPowSpec {
         *hasher.finalize().as_bytes()
     }
 
-    fn function(&self, tensors: &[Tensor], _device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
+    fn function(&self, tensors: &[Tensor]) -> Result<Tensor> {
         Python::with_gil(|py| {
             let pow = PY_UTILS
                 .pow
@@ -597,6 +804,162 @@ pub fn pow(base: CircuitRc, exponent: CircuitRc, name: Option<Name>) -> Result<G
     )
 }
 
+#[pyclass]
+#[derive(Debug, Clone)]
+pub struct GeneralFunctionMultinomialSpec {
+    #[pyo3(get)]
+    replacement: bool,
+    shape: Shape,
+}
+
+#[pymethods]
+impl GeneralFunctionMultinomialSpec {
+    #[getter]
+    #[pyo3(name = "shape")]
+    fn py_shape(&self) -> PyShape {
+        PyShape(self.shape.clone())
+    }
+}
+
+simple_from!(|x: GeneralFunctionMultinomialSpec| -> GeneralFunctionSpec {
+    GeneralFunctionSpec::Multinomial(x)
+});
+
+impl ToPyObject for GeneralFunctionMultinomialSpec {
+    fn to_object(&self, py: Python<'_>) -> PyObject {
+        self.clone().into_py(py)
+    }
+}
+
+fn sample_multinomial(
+    probs: Tensor,
+    shape: Shape,
+    replacement: bool,
+    seed_tensor: Tensor,
+) -> Result<Tensor, PyErr> {
+    Python::with_gil(|py| {
+        PY_UTILS
+            .random_indices
+            .call(py, (probs, shape.clone(), replacement, seed_tensor), None)?
+            .extract::<Tensor>(py)
+    })
+}
+
+impl SpecTrait for GeneralFunctionMultinomialSpec {
+    fn compute_hash(&self) -> HashBytes {
+        let mut hasher = blake3::Hasher::new();
+        hasher.update(uuid!("1f39ec5e-d3f8-4706-b075-1612fd83eef8").as_bytes());
+        hasher.update(&[self.replacement as u8]);
+        for l in &self.shape {
+            hasher.update(&l.to_le_bytes());
+        }
+        *hasher.finalize().as_bytes()
+    }
+
+    fn function(&self, tensors: &[Tensor]) -> Result<Tensor> {
+        Ok(sample_multinomial(
+            tensors[0].clone(),
+            self.shape.clone(),
+            self.replacement,
+            tensors[1].clone(),
+        )?)
+    }
+
+    fn get_shape_info(&self, shapes: &[Shape]) -> Result<GeneralFunctionShapeInfo> {
+        if shapes.len() != 2 {
+            bail!(GeneralFunctionShapeError::WrongNumShapes {
+                got: shapes.len(),
+                expected: 2
+            });
+        }
+
+        let (probs_shape, seed_shape) = (&shapes[0], &shapes[1]);
+        // TODO: improve
+        if !seed_shape.is_empty() {
+            bail!("seed input must be zero-dim (got shape={seed_shape:?})")
+        }
+        if probs_shape.is_empty() {
+            bail!("probs must be >=1d")
+        }
+
+        Ok(GeneralFunctionShapeInfo {
+            shape: probs_shape[..probs_shape.len() - 1]
+                .iter()
+                .chain(&self.shape)
+                .cloned()
+                .collect(),
+            num_non_batchable_output_dims: self.shape.len() as u8,
+            input_batchability: vec![false, false], /* note: batching over probs has right shape, but due to rng, doesn't keep values correctly */
+        })
+    }
+
+    fn name(&self) -> &'static str {
+        "multinomial"
+    }
+
+    fn is_official(&self) -> bool {
+        true
+    }
+
+    fn get_device_dtype_override(
+        &self,
+        device_dtypes: &[TorchDeviceDtypeOp],
+    ) -> Option<Result<TorchDeviceDtypeOp>> {
+        let (probs_dd, seed_dd) = (device_dtypes[0], device_dtypes[1]);
+        if probs_dd
+            .dtype
+            .map(|dtype| !dtype.is_floating_point())
+            .unwrap_or(false)
+        {
+            return Some(Err(anyhow!("probs dtype not floating point")));
+        }
+        if seed_dd
+            .dtype
+            .map(|dtype| dtype != TorchDtype::int64)
+            .unwrap_or(false)
+        {
+            return Some(Err(anyhow!(
+                "seed should be int64 (we could support other integers if we wanted)"
+            )));
+        }
+        if seed_dd
+            .device
+            .map(|device| device != TorchDevice::Cpu)
+            .unwrap_or(false)
+        {
+            return Some(Err(anyhow!("seed should be on cpu (for now at least)")));
+        }
+
+        Some(Ok(TorchDeviceDtypeOp {
+            device: probs_dd.device,
+            dtype: Some(TorchDtype::int64),
+        }))
+    }
+
+    fn serialize(&self) -> Result<Option<String>> {
+        Ok(Some(format!(
+            "multinomial{}_{:?}",
+            if !self.replacement { "_no_replace" } else { "" },
+            self.shape
+        )))
+    }
+}
+
+#[pyfunction(replacement = "true", name = "None")]
+pub fn multinomial(
+    probs: CircuitRc,
+    seed: CircuitRc,
+    shape: Shape,
+    replacement: bool,
+    name: Option<Name>,
+) -> Result<GeneralFunction> {
+    GeneralFunction::try_new(
+        vec![probs, seed],
+        GeneralFunctionSpec::Multinomial(GeneralFunctionMultinomialSpec { replacement, shape }),
+        name,
+    )
+}
+
 enum PyModuleLocator {
     Path(String),
     Module(String),
@@ -748,7 +1111,7 @@ impl SpecTrait for Box<PyWrap> {
         self.hash
     }
 
-    fn function(&self, tensors: &[Tensor], device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
+    fn function(&self, tensors: &[Tensor]) -> Result<Tensor> {
         let result: Tensor = Python::with_gil(|py| {
             Ok::<rr_util::py_types::Tensor, anyhow::Error>(
                 self.ob
@@ -760,8 +1123,6 @@ impl SpecTrait for Box<PyWrap> {
                     .extract(py)?,
             )
         })?;
-        TorchDeviceDtypeOp::from(TorchDeviceDtype::from_tensor(&result))
-            .combine(TorchDeviceDtypeOp::from(device_dtype.clone()))?;
         Ok(result)
     }
 
@@ -821,8 +1182,10 @@ impl SpecTrait for Box<PyWrap> {
 pub enum GeneralFunctionSpec {
     Simple(GeneralFunctionSimpleSpec),
     Index(GeneralFunctionIndexSpec),
+    ExplicitIndex(GeneralFunctionExplicitIndexSpec),
     SetDD(GeneralFunctionSetDDSpec),
     Pow(GeneralFunctionPowSpec),
+    Multinomial(GeneralFunctionMultinomialSpec),
     Py(Box<PyWrap>), // boxed bc it was long pole for Circuit size
 }
 
@@ -832,8 +1195,10 @@ impl GeneralFunctionSpec {
         match self {
             Self::Simple(x) => x,
             Self::Index(x) => x,
+            Self::ExplicitIndex(x) => x,
             Self::SetDD(x) => x,
             Self::Pow(x) => x,
+            Self::Multinomial(x) => x,
             Self::Py(x) => x,
         }
     }
@@ -854,8 +1219,8 @@ impl SpecTrait for GeneralFunctionSpec {
     fn compute_hash(&self) -> HashBytes {
         self.as_trait_obj().compute_hash()
     }
-    fn function(&self, tensors: &[Tensor], device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
-        self.as_trait_obj().function(tensors, device_dtype)
+    fn function(&self, tensors: &[Tensor]) -> Result<Tensor> {
+        self.as_trait_obj().function(tensors)
     }
     fn get_shape_info(&self, shapes: &[Shape]) -> Result<GeneralFunctionShapeInfo> {
         self.as_trait_obj().get_shape_info(shapes)
@@ -1000,8 +1365,8 @@ impl CircuitNode for GeneralFunction {
             .collect()
     }
 
-    fn eval_tensors(&self, tensors: &[Tensor], device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
-        self.spec.function(tensors, device_dtype)
+    fn eval_tensors(&self, tensors: &[Tensor]) -> Result<Tensor> {
+        self.spec.function(tensors)
     }
 }
 
@@ -1076,8 +1441,14 @@ impl GeneralFunction {
         name: Option<Name>,
     ) -> Result<Option<Self>> {
         // parse index case
-        static RE_GEN_INDEX: Lazy<Regex> =
-            Lazy::new(|| Regex::new(r"^gen_index_at_(-?\d+)(_batch_x)?(_c)?$").unwrap());
+        static RE_MULTINOMIAL: Lazy<Regex> = Lazy::new(|| {
+            Regex::new(r"^multinomial(_no_replace)?_\[\s*((?:\d+\s*,\s*)*(?:\d+\s*)?)\]$").unwrap()
+        });
+        static RE_GEN_INDEX: Lazy<Regex> = Lazy::new(|| {
+            Regex::new(r"^gen_index_at_(-?\d+)(_batch_x)?(_no_batch_index)?(_c)?$").unwrap()
+        });
+        static RE_EXPLICIT_INDEX: Lazy<Regex> =
+            Lazy::new(|| Regex::new(r"^explicit_index_at_(-?\d+)_x_non_b_(\d+)(_c)?$").unwrap());
         static RE_CAST: Lazy<Regex> = Lazy::new(|| {
             Regex::new(r"^cast_from_\{device:([a-zA-Z0-9]+),dtype:([a-zA-Z0-9]+)\}_to_\{device:([a-zA-Z0-9]+),dtype:([a-zA-Z0-9]+)\}$").unwrap()
         });
@@ -1089,10 +1460,51 @@ impl GeneralFunction {
                 .parse()
                 .context("failed to parse index_dim for gen_index_at")?;
             let batch_x = re_captures.get(2).is_some();
-            let check_index_ints = re_captures.get(3).is_some();
+            let batch_index = !re_captures.get(3).is_some();
+            let check_index_ints = re_captures.get(4).is_some();
             GeneralFunctionIndexSpec {
                 index_dim,
                 batch_x,
+                batch_index,
+                check_index_ints,
+            }
+            .into()
+        } else if let Some(re_captures) = RE_MULTINOMIAL.captures(&spec_name) {
+            let mut axis_strs: Vec<_> = re_captures
+                .get(2)
+                .unwrap()
+                .as_str()
+                .split(',')
+                .map(|z| z.trim())
+                .collect();
+            if axis_strs.last() == Some(&"") {
+                // allow last axis to be empty due to trailing comma
+                // (Regex guarantees that only last axis has this I think, but better to be clear)
+                axis_strs.pop();
+            }
+            let shape = axis_strs.into_iter().map(|x| x.parse().unwrap()).collect();
+            GeneralFunctionMultinomialSpec {
+                replacement: !re_captures.get(1).is_some(),
+                shape,
+            }
+            .into()
+        } else if let Some(re_captures) = RE_EXPLICIT_INDEX.captures(&spec_name) {
+            let index_dim = re_captures
+                .get(1)
+                .unwrap()
+                .as_str()
+                .parse()
+                .context("failed to parse index_dim for explicit_index_at")?;
+            let x_non_batch_dims = re_captures
+                .get(2)
+                .unwrap()
+                .as_str()
+                .parse()
+                .context("failed to parse x_non_batch_dims for explicit_index_at")?;
+            let check_index_ints = re_captures.get(3).is_some();
+            GeneralFunctionExplicitIndexSpec {
+                index_dim,
+                x_non_batch_dims,
                 check_index_ints,
             }
             .into()
@@ -1154,23 +1566,47 @@ impl GeneralFunction {
     }
 
     #[staticmethod]
-    #[args(batch_x = "false", check_index_ints = "true", name = "None")]
+    #[args(
+        batch_x = "false",
+        batch_index = "true",
+        check_index_ints = "true",
+        name = "None"
+    )]
     pub fn gen_index(
         x: CircuitRc,
         index: CircuitRc,
         index_dim: i64,
         batch_x: bool,
+        batch_index: bool,
         check_index_ints: bool,
         name: Option<Name>,
     ) -> Result<Self> {
         let spec = GeneralFunctionIndexSpec {
             index_dim,
             batch_x,
+            batch_index,
             check_index_ints,
         }
         .into();
         Self::try_new(vec![x, index], spec, name)
     }
+
+    #[staticmethod]
+    #[args(check_index_ints = "true", name = "None")]
+    pub fn explicit_index(
+        x: CircuitRc,
+        index: CircuitRc,
+        index_dim: i64,
+        x_non_batch_dims: usize,
+        check_index_ints: bool,
+        name: Option<Name>,
+    ) -> Result<Self> {
+        let spec =
+            GeneralFunctionExplicitIndexSpec::new(index_dim, x_non_batch_dims, check_index_ints)?
+                .into();
+        Self::try_new(vec![x, index], spec, name)
+    }
+
     #[staticmethod]
     #[args(
         input_required_compatibility = "Default::default()",
@@ -1459,7 +1895,7 @@ impl GeneralFunctionSpecTester {
                     .collect();
 
                 let out_tensor = spec
-                    .function(&tensors, &TorchDeviceDtype::default())
+                    .function(&tensors)
                     .context("failed to evaluate for test")?;
                 if out_tensor.shape() != &shape {
                     bail!(
@@ -1495,7 +1931,6 @@ impl GeneralFunctionSpecTester {
                             .zip(&input_batchability)
                             .map(|(x, &b)| if b { run_idx(x.clone()) } else { x.clone() })
                             .collect::<Vec<_>>(),
-                        &Default::default(),
                     )
                     .context("failed to evaluate for test")?;
 
@@ -1644,18 +2079,28 @@ fn test_index_general_function() -> Result<()> {
     let mut rng = rand::thread_rng();
     for index_dim in -2..2 {
         for batch_x in [false, true] {
-            let spec = GeneralFunctionIndexSpec {
-                index_dim,
-                batch_x,
-                check_index_ints: true,
+            for batch_index in [false, true] {
+                let spec = GeneralFunctionIndexSpec {
+                    index_dim,
+                    batch_x,
+                    batch_index,
+                    check_index_ints: true,
+                }
+                .into();
+                if !batch_index && index_dim >= 0 {
+                    continue;
+                }
+                if !batch_x && !batch_index {
+                    continue;
+                }
+                tester.test_many_shapes(spec)?;
             }
-            .into();
-            tester.test_many_shapes(spec)?;
         }
 
         let spec: GeneralFunctionSpec = GeneralFunctionIndexSpec {
             index_dim,
             batch_x: true,
+            batch_index: true,
             check_index_ints: true,
         }
         .into();
@@ -1735,4 +2180,12 @@ pub enum GeneralFunctionShapeError {
         index_shape: Shape,
         batch_x: bool,
     },
+
+    #[error("x_shape={x_shape:?} index_shape={index_shape:?} index_dim={index_dim} x_non_batch_dims={x_non_batch_dims} ({e_name})")]
+    ExplicitIndexShapeInvalid {
+        x_shape: Shape,
+        index_shape: Shape,
+        index_dim: i64,
+        x_non_batch_dims: usize,
+    },
 }
