@@ -37,7 +37,8 @@ use rr_util::{
     eq_by_big_hash::EqByBigHash,
     name::Name,
     py_types::{
-        reduction_to_ints, use_rust_comp, PyCallable, PyOpAtAxes, PyShape, Tensor, PY_CIRCUIT_ITEMS,
+        reduction_to_ints, use_rust_comp, PyCallable, PyOpAtAxes, PyShape, Tensor,
+        PY_CIRCUIT_ITEMS, PY_UTILS,
     },
     pycall, pycallable, python_error_exception,
     rearrange_spec::{OpSize, PyCountsAtAxes, RInnerInts, RearrangeSpec},
@@ -80,7 +81,7 @@ pub mod set_of_circuits;
 pub mod set_symbolic_shape;
 pub mod variable_nodes;
 
-mod circuit_node_private;
+pub mod circuit_node_private;
 use circuit_node_private::*;
 
 use self::circuit_utils::total_flops;
@@ -153,7 +154,7 @@ pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash {
         BigUint::from(0usize)
     }
 
-    fn eval_tensors(&self, tensors: &[Tensor], device_dtype: &TorchDeviceDtype) -> Result<Tensor>;
+    fn eval_tensors(&self, tensors: &[Tensor]) -> Result<Tensor>;
 
     fn c(self) -> Circuit;
     fn rc(self) -> CircuitRc;
@@ -1224,8 +1225,8 @@ impl<T: CircuitNodeDefer> CircuitNode for T {
         self.as_trait_obj().self_flops()
     }
 
-    fn eval_tensors(&self, tensors: &[Tensor], device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
-        self.as_trait_obj().eval_tensors(tensors, device_dtype)
+    fn eval_tensors(&self, tensors: &[Tensor]) -> Result<Tensor> {
+        self.as_trait_obj().eval_tensors(tensors)
     }
 
     fn c(self) -> Circuit {
@@ -1615,9 +1616,21 @@ pub fn make_children_zero<T: CircuitNode>(circuit: &T) -> T {
         Scalar::new(0.0, child.info().shape.clone(), child.info().name).rc()
     })
 }
-
+#[pyfunction]
 pub fn evaluate(circ: CircuitRc) -> Result<Tensor> {
-    evaluate_device_dtype(circ, Default::default())
+    check_evaluable(circ.clone())?;
+    let circ = substitute_all_modules(circ);
+    #[apply(cached_lambda)]
+    #[key(circ.info().hash, HashBytes)]
+    #[use_try]
+    fn recurse(circ: CircuitRc) -> Result<Tensor> {
+        let child_tensors: Result<Vec<Tensor>> = circ.children().map(recurse).collect();
+        let child_tensors = child_tensors?;
+
+        circ.eval_tensors(&child_tensors)
+    }
+
+    recurse(circ)
 }
 
 #[pyfunction]
@@ -1635,30 +1648,25 @@ pub fn check_evaluable(circuit: CircuitRc) -> Result<()> {
     Ok(())
 }
 
-#[pyfunction(dtype_device = "TorchDeviceDtypeOp::NONE")]
-#[pyo3(name = "evaluate")]
-pub fn evaluate_device_dtype(circ: CircuitRc, device_dtype: TorchDeviceDtypeOp) -> Result<Tensor> {
-    check_evaluable(circ.clone())?;
-    let device_dtype = device_dtype
-        .clone()
-        .combine(circ.info().device_dtype.clone())
-        .map_err(|_err| TensorEvalError::DeviceDtypeError {
-            circ: circ.info().device_dtype.clone(),
-            passed: device_dtype.clone(),
-        })?
-        .unwrap_or_defaults();
-    let circ = substitute_all_modules(circ);
-    #[apply(cached_lambda)]
-    #[key(circ.info().hash, HashBytes)]
-    #[use_try]
-    fn recurse(circ: CircuitRc) -> Result<Tensor> {
-        let child_tensors: Result<Vec<Tensor>> = circ.children().map(recurse).collect();
-        let child_tensors = child_tensors?;
-
-        circ.eval_tensors(&child_tensors, &device_dtype)
+pub fn check_combine_evaluate_device_dtype(
+    circ: CircuitRc,
+    device_dtype: TorchDeviceDtypeOp,
+) -> Result<TorchDeviceDtype> {
+    if device_dtype
+        .dtype
+        .map(|x| !x.is_floating_point())
+        .unwrap_or(false)
+    {
+        bail!("evaluate dtype is used for scalars and other base cases and must be floating point");
     }
-
-    recurse(circ)
+    let out = TorchDeviceDtypeOp {
+        device: device_dtype.device.or(circ.info().device_dtype.device),
+        dtype: device_dtype
+            .dtype
+            .or(circ.info().device_dtype.only_keep_floating().dtype),
+    }
+    .unwrap_or_defaults();
+    Ok(out)
 }
 
 #[apply(pycallable)]
@@ -1950,14 +1958,12 @@ where
     result
 }
 
-pub fn evaluate_fn_uncached(circ: CircuitRc, device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
-    let child_tensors: Result<Vec<Tensor>> = circ
-        .children()
-        .map(|x| evaluate_fn_uncached(x, device_dtype))
-        .collect();
+pub fn evaluate_fn_uncached(circ: CircuitRc) -> Result<Tensor> {
+    let child_tensors: Result<Vec<Tensor>> =
+        circ.children().map(|x| evaluate_fn_uncached(x)).collect();
     let child_tensors = child_tensors?;
 
-    circ.eval_tensors(&child_tensors, device_dtype)
+    circ.eval_tensors(&child_tensors)
 }
 
 impl IntoPy<PyObject> for CircuitRc {
@@ -2221,9 +2227,8 @@ impl PyCircuitBase {
         circuit_rust_to_py(self.0.clone())
     }
 
-    #[args(device_dtype = "Default::default()")]
-    fn evaluate(&self, device_dtype: TorchDeviceDtypeOp) -> Result<Tensor> {
-        evaluate_device_dtype(self.0.clone(), device_dtype)
+    fn evaluate(&self) -> Result<Tensor> {
+        evaluate(self.0.clone())
     }
 
     fn map_children_enumerate(&self, f: PyCallable) -> Result<CircuitRc> {
@@ -2373,6 +2378,13 @@ impl PyCircuitBase {
     fn rearrange_str(&self, string: &str, name: Option<Name>) -> Result<Rearrange> {
         self.0.rearrange_str(string, name)
     }
+    fn enforce_dtype_device(
+        &self,
+        device_dtype: TorchDeviceDtypeOp,
+        name: Option<Name>,
+    ) -> GeneralFunction {
+        enforce_dtype_device(self.0.clone(), device_dtype, name)
+    }
     pub fn is_irreducible_node(&self) -> bool {
         self.0.is_irreducible_node()
     }
@@ -2480,3 +2492,46 @@ pub mod prelude {
 pub fn circuit_rust_to_py(circ: CircuitRc) -> PyObject {
     Python::with_gil(|py| PY_CIRCUIT_ITEMS.rust_to_py.call(py, (circ,), None).unwrap())
 }
+
+/// casts tensors whith None device/dtype to specified device/dtype
+pub fn upcast_tensors(
+    tensors: &[Tensor],
+    device_dtypes: &[TorchDeviceDtypeOp],
+    out_device_dtype: TorchDeviceDtypeOp,
+) -> Vec<Tensor> {
+    zip(tensors, device_dtypes)
+        .map(|(t, dd)| {
+            if *dd != out_device_dtype {
+                pycall!(
+                    PY_UTILS.cast_tensor,
+                    (t.clone(), out_device_dtype.unwrap_or_defaults())
+                )
+            } else {
+                t.clone()
+            }
+        })
+        .collect()
+}
+
+/// alias for new_cast with same input, output constraint
+pub fn enforce_dtype_device(
+    circ: CircuitRc,
+    device_dtype: TorchDeviceDtypeOp,
+    name: Option<Name>,
+) -> GeneralFunction {
+    GeneralFunction::new_cast(circ, device_dtype, device_dtype, name).unwrap()
+}
+
+#[macro_export]
+macro_rules! upcast_tensors_for_eval {
+    ($selfy:ident, $tensory:ident) => {
+        $crate::upcast_tensors(
+            $tensory,
+            &($selfy
+                .children()
+                .map(|z| z.info().device_dtype)
+                .collect::<Vec<_>>()),
+            $selfy.info().device_dtype,
+        )
+    };
+}
