@@ -18,7 +18,7 @@ use rr_util::{
     pycall, python_error_exception,
     rearrange_spec::RearrangeSpec,
     symbolic_size::SymbolicSizeProduct,
-    tensor_util::{right_align_shapes, Shape, TorchDeviceDtype},
+    tensor_util::{right_align_shapes, Shape},
     util::{arc_unwrap_or_clone, counts_g_1, is_unique},
     IndexSet,
 };
@@ -1093,7 +1093,6 @@ impl CircuitNode for Module {
     fn eval_tensors(
         &self,
         _tensors: &[rr_util::py_types::Tensor],
-        _device_dtype: &TorchDeviceDtype,
     ) -> Result<rr_util::py_types::Tensor> {
         bail!(TensorEvalError::ModulesCantBeDirectlyEvalutedInternal {
             module: self.clone(),
