@@ -13,8 +13,7 @@ use rr_util::{
     symbolic_size::{SymbolicSizeProduct, SIZE_PROD_MATCH},
     tensor_db::ensure_all_tensors_local,
     tensor_util::{
-        check_canon_idxs, parse_numeric, ParseError, PyParseError, Shape, TensorIndex,
-        TorchDeviceDtypeOp,
+        parse_numeric, ParseError, PyParseError, Shape, TensorIndex, TorchDeviceDtypeOp,
     },
     timed,
     util::{counts_g_1, flip_op_result, is_unique, NamedAxes},
@@ -55,7 +54,7 @@ impl Default for Parser {
         Self {
             reference_circuits: Default::default(),
             tensors_as_random: false,
-            tensors_as_random_device_dtype: TorchDeviceDtypeOp::NONE,
+            tensors_as_random_device_dtype: TorchDeviceDtypeOp::TENSOR_DEFAULT,
             allow_hash_with_random: false,
             on_repeat_check_info_same: true,
             module_check_all_inputs_used: true,
