@@ -9,7 +9,7 @@ use std::{
 
 use anyhow::{anyhow, bail, Context, Error, Result};
 use itertools::Itertools;
-use num_bigint::{BigInt, BigUint};
+use num_bigint::BigUint;
 use once_cell::unsync::Lazy;
 use pyo3::{once_cell::GILLazy, prelude::*};
 use rr_util::{
@@ -46,6 +46,12 @@ const NAME_TO_COLOR: Lazy<HashMap<String, usize>> = Lazy::new(|| {
         .map(|(a, b)| (a.to_owned(), b)),
     )
 });
+const COLOR_TO_NAME: Lazy<HashMap<usize, String>> = Lazy::new(|| {
+    NAME_TO_COLOR
+        .iter()
+        .map(|(a, b)| (b.clone(), a.clone()))
+        .collect()
+});
 
 #[derive(Clone, Copy, Hash, PartialEq, Eq, PartialOrd, Ord)]
 pub struct CliColor(Option<usize>);
@@ -58,6 +64,15 @@ impl CliColor {
             .ok_or_else(|| anyhow!("unknown color name {}", string))
             .map(|x| CliColor(Some(*x)))
     }
+    pub fn to_string(&self) -> String {
+        if let Some(s) = self.0 {
+            if let Some(ctn) = COLOR_TO_NAME.get(&s) {
+                return ctn.clone();
+            }
+            return s.to_string();
+        }
+        "None".to_owned()
+    }
     pub fn new(int: usize) -> Self {
         CliColor(Some(int % COLOR_CODES.len()))
     }
@@ -103,6 +118,12 @@ impl Deref for CliColor {
     }
 }
 
+impl std::fmt::Display for CliColor {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        write!(f, "{}", self.to_string())
+    }
+}
+
 #[macro_export]
 macro_rules! clicolor {
     ($name:ident) => {
