@@ -3,12 +3,7 @@ use std::collections::BTreeMap;
 use anyhow::{bail, Result};
 use macro_rules_attribute::apply;
 use pyo3::prelude::*;
-use rr_util::{
-    name::Name,
-    py_types::Tensor,
-    sv,
-    tensor_util::{Shape, TorchDeviceDtype},
-};
+use rr_util::{name::Name, py_types::Tensor, sv, tensor_util::Shape};
 use uuid::Uuid;
 
 use crate::{
@@ -97,7 +92,7 @@ impl CircuitNode for Tag {
         ))
     }
 
-    fn eval_tensors(&self, tensors: &[Tensor], _device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
+    fn eval_tensors(&self, tensors: &[Tensor]) -> Result<Tensor> {
         assert_eq!(tensors.len(), 1);
         Ok(tensors[0].clone())
     }
@@ -230,11 +225,7 @@ impl CircuitNode for DiscreteVar {
         )
     }
 
-    fn eval_tensors(
-        &self,
-        _tensors: &[Tensor],
-        _device_dtype: &TorchDeviceDtype,
-    ) -> Result<Tensor> {
+    fn eval_tensors(&self, _tensors: &[Tensor]) -> Result<Tensor> {
         Err(TensorEvalError::NotExplicitlyComputableInternal {
             circuit: self.crc(),
         }
@@ -413,11 +404,7 @@ impl CircuitNode for StoredCumulantVar {
         )
     }
 
-    fn eval_tensors(
-        &self,
-        _tensors: &[Tensor],
-        _device_dtype: &TorchDeviceDtype,
-    ) -> Result<Tensor> {
+    fn eval_tensors(&self, _tensors: &[Tensor]) -> Result<Tensor> {
         Err(TensorEvalError::NotExplicitlyComputableInternal {
             circuit: self.crc(),
         }
