@@ -2,7 +2,7 @@ use anyhow::Result;
 use circuit_base::{deep_map_preorder, deep_map_preorder_unwrap, prelude::*, visit_circuit};
 use macro_rules_attribute::apply;
 use pyo3::{pyfunction, PyObject};
-use rr_util::{pycall, pycallable};
+use rr_util::{py_types::PyCallable, pycall, pycallable};
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
 
 #[apply(pycallable)]
@@ -33,8 +33,8 @@ pub fn replace_nodes_py(circuit: CircuitRc, map: HashMap<CircuitRc, CircuitRc>)
 #[pyo3(name = "update_nodes")]
 pub fn update_nodes_py(
     circuit: CircuitRc,
-    matcher: PyObject,
-    updater: PyObject,
+    matcher: PyCallable,
+    updater: PyCallable,
 ) -> Result<CircuitRc> {
     let nodes = filter_nodes_py(circuit.clone(), matcher)?;
     deep_map_preorder(circuit, |x| {
