@@ -16,7 +16,7 @@ use rr_util::{
     py_types::{is_python_running, Tensor, PY_UTILS},
     python_println,
     repr::RustRepr,
-    tensor_util::{Shape, TorchDeviceDtype},
+    tensor_util::Shape,
     timed,
     util::{apply_fn_until_same, HashBytes},
 };
@@ -80,8 +80,6 @@ pub struct OptimizationSettings {
     #[pyo3(get, set)]
     pub optimization_parallelism: Option<usize>,
     #[pyo3(get, set)]
-    pub device_dtype: Option<TorchDeviceDtype>,
-    #[pyo3(get, set)]
     pub keep_all_names: bool,
 }
 
@@ -106,7 +104,6 @@ impl Default for OptimizationSettings {
             log_slow_einsums: false,
             save_circ: false,
             optimization_parallelism: Some(8),
-            device_dtype: None,
             keep_all_names: false,
         }
     }
@@ -134,7 +131,6 @@ impl OptimizationSettings {
         log_slow_einsums = "OptimizationSettings::default().log_slow_einsums",
         save_circ = "OptimizationSettings::default().save_circ",
         optimization_parallelism = "None",
-        device_dtype = "None",
         keep_all_names = "OptimizationSettings::default().keep_all_names"
     )]
     fn new(
@@ -156,7 +152,6 @@ impl OptimizationSettings {
         log_slow_einsums: bool,
         save_circ: bool,
         optimization_parallelism: Option<usize>,
-        device_dtype: Option<TorchDeviceDtype>,
         keep_all_names: bool,
     ) -> Self {
         Self {
@@ -178,7 +173,6 @@ impl OptimizationSettings {
             log_slow_einsums,
             save_circ,
             optimization_parallelism,
-            device_dtype,
             keep_all_names,
         }
     }
@@ -358,7 +352,7 @@ pub fn optimize_to_schedule(
     let optimized_circuit = timed!(optimize_circuit(circuit, context), 10, verbose >= 1)?;
     // return evaluate_fn(&*optimized_circuit).unwrap();
     let schedule = if settings.scheduling_naive {
-        circuit_to_schedule_naive_toposort(optimized_circuit, context.settings.device_dtype)?
+        circuit_to_schedule_naive_toposort(optimized_circuit)?
     } else {
         timed!(
             circuit_to_schedule(optimized_circuit, context),
