@@ -1,4 +1,4 @@
-use circuit_base::{deep_map_op, deep_map_op_context, CircuitNode, CircuitRc, Tag};
+use circuit_base::{deep_map_op_context, CircuitNode, CircuitRc, Tag};
 use pyo3::prelude::*;
 use uuid::Uuid;
 
