@@ -95,7 +95,7 @@ pub fn einsum_pull_concat(einsum: &Einsum) -> Option<CircuitRc> {
                     })
                     .collect();
 
-                Einsum::nrc(einsum_args, einsum.out_axes.clone(), einsum.name_cloned())
+                Einsum::nrc(einsum_args, einsum.out_axes.clone(), einsum.info().name)
             })
             .collect();
             Add::nrc(summands, None)
@@ -221,7 +221,7 @@ pub fn concat_fuse(concat: &Concat) -> Option<Concat> {
     if !did_anything {
         return None;
     }
-    Some(Concat::new(new_operands, concat.axis, concat.name_cloned()))
+    Some(Concat::new(new_operands, concat.axis, concat.info().name))
 }
 
 #[pyfunction]
@@ -243,7 +243,7 @@ pub fn concat_drop_size_zero(concat: &Concat) -> Option<Concat> {
     if !did_anything {
         return None;
     }
-    Some(Concat::new(new_nodes, concat.axis, concat.name_cloned()))
+    Some(Concat::new(new_nodes, concat.axis, concat.info().name))
 }
 
 #[pyfunction]
@@ -306,9 +306,9 @@ pub fn index_concat_drop_unreached(index: &Index) -> Option<CircuitRc> {
     let mut new_tensor_index = index.index.clone();
     new_tensor_index.0[concat.axis] = new_index;
     Some(Index::nrc(
-        Concat::nrc(new_operands, concat.axis, concat.name_cloned()),
+        Concat::nrc(new_operands, concat.axis, concat.info().name),
         new_tensor_index,
-        index.name_cloned(),
+        index.info().name,
     ))
 }
 
@@ -388,7 +388,7 @@ pub fn concat_elim_split(circuit: &Concat) -> Option<Index> {
     Some(Index::new(
         child,
         TensorIndex(new_index),
-        circuit.name_cloned(),
+        circuit.info().name,
     ))
 }
 
