@@ -1,10 +1,6 @@
-use std::{
-    cell::{Cell, UnsafeCell},
-    simd::{u64x8, u8x8, SimdPartialEq},
-};
+use std::simd::{u64x8, u8x8, SimdPartialEq};
 
 use anyhow::{bail, Result};
-use cached::proc_macro::cached;
 use circuit_base::{
     deep_map_op_context, deep_map_preorder_unwrap, deep_map_unwrap, on_circuit_names, prelude::*,
     visit_circuit_unwrap, Add, Index,
@@ -32,8 +28,8 @@ use crate::{
     },
     diag_rewrite::{add_pull_diags, einsum_push_down_trace},
     generalfunction_rewrite::{
-        generalfunction_evaluate_simple, generalfunction_merge_inverses,
-        generalfunction_special_case_simplification,
+        generalfunction_evaluate_simple, generalfunction_gen_index_const_to_index,
+        generalfunction_merge_inverses, generalfunction_special_case_simplification,
     },
     module_rewrite::{elim_empty_module, elim_no_input_module},
     nb_rewrites::{add_elim_removable_axes_weak, einsum_elim_removable_axes_weak},
@@ -210,6 +206,7 @@ fn all_simp_fns_raw() -> SimpFns {
             l_wrap!(generalfunction_merge_inverses),
             l_wrap!(generalfunction_special_case_simplification),
             l_wrap!(generalfunction_evaluate_simple),
+            l_wrap!(generalfunction_gen_index_const_to_index),
         ],
         scatter: vec![
             l_wrap!(scatter_elim_identity),
