@@ -1,18 +1,17 @@
 use circuit_base::{
-    evaluate_device_dtype,
+    evaluate,
     generalfunction::{SpecTrait, OFFICIAL_GENERALFUNCTION_INVERSES},
-    Array, Circuit, CircuitNode, CircuitRc, GeneralFunction, PyCircuitBase, Scalar,
+    Circuit, CircuitNode, CircuitRc, GeneralFunction, GeneralFunctionSpec, Index, Scalar,
 };
 use pyo3::prelude::*;
 use rr_util::{
-    py_types::scalar_to_tensor,
-    pycall, sv,
+    sv,
     symbolic_size::SymbolicSizeProduct,
-    tensor_util::{TorchDeviceDtype, TorchDeviceDtypeOp, TorchDtype},
+    tensor_util::{TensorAxisIndex, TensorIndex},
 };
 #[pyfunction]
 pub fn generalfunction_merge_inverses(circuit: &GeneralFunction) -> Option<CircuitRc> {
-    if circuit.nodes.len()==1 && circuit.spec.is_official() && let Circuit::GeneralFunction(inner)=&**circuit.nodes[0] && inner.spec.is_official() && OFFICIAL_GENERALFUNCTION_INVERSES.iter().any(|(a,b)|*a==&circuit.spec.name()&&*b==&inner.spec.name()) {
+    if circuit.nodes.len()==1 && circuit.spec.is_official() && let Circuit::GeneralFunction(inner)=&**circuit.nodes[0] && inner.spec.is_official() && OFFICIAL_GENERALFUNCTION_INVERSES.iter().any(|(a,b)|a==&circuit.spec.name()&&b==&inner.spec.name()) {
         return Some(inner.nodes[0].clone())
     }
     None
@@ -30,7 +29,7 @@ pub fn generalfunction_special_case_simplification(circuit: &GeneralFunction) ->
                     return Some(Scalar::nrc(
                         scalar,
                         circuit.info().shape.clone(),
-                        circuit.name_cloned(),
+                        circuit.info().name,
                     ));
                 }
             }
@@ -42,7 +41,7 @@ pub fn generalfunction_special_case_simplification(circuit: &GeneralFunction) ->
                     return Some(Scalar::nrc(
                         scalar,
                         circuit.info().shape.clone(),
-                        circuit.name_cloned(),
+                        circuit.info().name,
                     ));
                 }
             }
@@ -56,7 +55,7 @@ pub fn generalfunction_special_case_simplification(circuit: &GeneralFunction) ->
                             .iter()
                             .cloned()
                             .collect(),
-                        circuit.name_cloned(),
+                        circuit.info().name,
                     ));
                 }
             }
@@ -77,23 +76,17 @@ pub fn generalfunction_evaluate_simple(node: &GeneralFunction) -> Option<Circuit
         return node.nodes[0].as_scalar().map(|_inner| {
             Python::with_gil(|py| {
                 Scalar::new(
-                    evaluate_device_dtype(
-                        node.crc(),
-                        TorchDeviceDtypeOp {
-                            device: None,
-                            dtype: Some(TorchDtype::float64),
-                        },
-                    )
-                    .unwrap()
-                    .tensor()
-                    .getattr(py, "item")
-                    .unwrap()
-                    .call(py, (), None)
-                    .unwrap()
-                    .extract(py)
-                    .unwrap(),
+                    evaluate(node.crc())
+                        .unwrap()
+                        .tensor()
+                        .getattr(py, "item")
+                        .unwrap()
+                        .call(py, (), None)
+                        .unwrap()
+                        .extract(py)
+                        .unwrap(),
                     sv![],
-                    node.name_cloned(),
+                    node.info().name,
                 )
                 .rc()
             })
@@ -101,3 +94,29 @@ pub fn generalfunction_evaluate_simple(node: &GeneralFunction) -> Option<Circuit
     }
     None
 }
+
+#[pyfunction]
+pub fn generalfunction_gen_index_const_to_index(node: &GeneralFunction) -> Option<CircuitRc> {
+    if let GeneralFunctionSpec::Index(i) = &node.spec
+        && let Circuit::Array(a) = &**node.nodes[1]
+        && !i.batch_index
+        && i.batch_x {
+        let x = &node.nodes[0];
+        let idx = if i.index_dim < 0 {
+            (i.index_dim % (a.ndim() as i64)) as usize
+        } else {
+            i.index_dim as usize
+        };
+        if a.ndim() == 1 {
+
+        return Some(Index::nrc(
+            x.clone(),
+            TensorIndex::new_single(TensorAxisIndex::Tensor(a.value.clone().try_into().unwrap()), idx, x.ndim()),
+            node.info().name
+        ))
+        } else {
+            return None
+        }
+    }
+    None
+}
