@@ -26,19 +26,22 @@ pub fn elim_no_input_module(module: &Module) -> Option<CircuitRc> {
 
 #[pyfunction(
     add_suffix_on_remove_unused = "true",
-    use_elim_no_input_module = "true"
+    use_elim_no_input_module = "true",
+    elim_empty = "false"
 )]
 #[pyo3(name = "deep_module_remove_unused_inputs")]
 pub fn py_deep_module_remove_unused_inputs(
     circuit: CircuitRc,
     add_suffix_on_remove_unused: bool,
     use_elim_no_input_module: bool,
+    elim_empty: bool,
 ) -> CircuitRc {
     deep_module_remove_unused_inputs(
         circuit,
         &mut Default::default(),
         add_suffix_on_remove_unused,
         use_elim_no_input_module,
+        elim_empty,
     )
 }
 
@@ -47,11 +50,15 @@ pub fn deep_module_remove_unused_inputs(
     cache: &mut HashMap<HashBytes, Option<CircuitRc>>,
     add_suffix_on_remove_unused: bool,
     use_elim_no_input_module: bool,
+    elim_empty: bool,
 ) -> CircuitRc {
     deep_map_op_context(
         circuit.clone(),
         &|c, _| {
             c.as_module().and_then(|m| {
+                if elim_empty && let Some(eem) = elim_empty_module(m) {
+                    return Some(eem);
+                }
                 module_remove_unused_inputs(
                     m,
                     add_suffix_on_remove_unused,
