@@ -6,8 +6,8 @@ use std::{
 use anyhow::{anyhow, bail, Context, Result};
 use circuit_base::{
     cached_circuit_properties::max_non_leaf_size,
+    circuit_node_private::CircuitNodePrivate,
     circuit_utils::{hash_to_node_non_free, is_definitely_view_on_child, toposort_circuit},
-    get_compatible_dtype,
     module::{any_children_with_symbolic_sizes, substitute_all_modules},
     prelude::*,
     print::{oom_fmt, PrintOptions},
@@ -230,7 +230,7 @@ impl Schedule {
             } else {
                 evaluate_schedule(self)?[&self.output_circuit.as_ref().unwrap().0].clone()
             };
-            if timed && self.device_dtype.device != TorchDevice::Cpu {
+            if timed && self.device_dtype.device.is_cuda() {
                 Python::with_gil(|py| {
                     PY_UTILS
                         .torch
@@ -362,6 +362,9 @@ impl Schedule {
             .evaluate_remote_many(remote_url, self.device_dtype.device.clone());
         Ok(out)
     }
+    fn __repr__(&self) -> String {
+        format!("{}", &self)
+    }
 }
 impl Schedule {
     pub fn deserialize(
@@ -389,6 +392,8 @@ impl Display for Schedule {
                                 + " "
                                 + &c.variant_string()
                                 + " "
+                                + &format!("{:?}", c.info().device_dtype.dtype)
+                                + " "
                                 + &c.children()
                                     .map(|x| get_child_key(x).to_string())
                                     .collect::<Vec<String>>()
@@ -547,7 +552,7 @@ pub fn evaluate_schedule(schedule: &Schedule) -> Result<HashMap<usize, Tensor>>
         schedule
             .scalars
             .iter()
-            .map(|x| (*x.0, x.1.eval_tensors(&[], &schedule.device_dtype).unwrap())),
+            .map(|x| (*x.0, x.1.eval_tensors(&[]).unwrap())),
     );
     for s in &schedule.instructions {
         let run = |_py: Python| match s {
@@ -562,7 +567,7 @@ pub fn evaluate_schedule(schedule: &Schedule) -> Result<HashMap<usize, Tensor>>
                     .iter()
                     .map(|child_key| live[child_key].clone())
                     .collect();
-                let result_err = circ.eval_tensors(&tensors, &schedule.device_dtype);
+                let result_err = circ.eval_tensors(&tensors);
                 if result_err.is_err() {
                     println!("errored evaluate");
                     circ.print().unwrap()
@@ -641,22 +646,13 @@ pub fn evaluate_schedule_adjust_numerical_scale(
                 || value_scale < settings.numerical_scale_min && value_scale != 0.0
             {
                 (
-                    scalar_to_tensor(
-                        s.value.signum(),
-                        s.info().shape.clone(),
-                        schedule.device_dtype.clone(),
-                    )
-                    .unwrap(),
+                    scalar_to_tensor(s.value.signum(), s.info().shape.clone(), Default::default())
+                        .unwrap(),
                     1.0 / value_scale,
                 )
             } else {
                 (
-                    scalar_to_tensor(
-                        s.value,
-                        s.info().shape.clone(),
-                        schedule.device_dtype.clone(),
-                    )
-                    .unwrap(),
+                    scalar_to_tensor(s.value, s.info().shape.clone(), Default::default()).unwrap(),
                     1.0,
                 )
             }
@@ -678,17 +674,13 @@ pub fn evaluate_schedule_adjust_numerical_scale(
                 let result = match &***circ {
                     Circuit::Einsum(_) => {
                         let new_scale = tensors_and_scales.iter().map(|x| x.1).product();
-                        clamp(&(
-                            circ.eval_tensors(&tensors, &schedule.device_dtype).unwrap(),
-                            new_scale,
-                        ))
+                        clamp(&(circ.eval_tensors(&tensors).unwrap(), new_scale))
                     }
                     Circuit::Add(_) | Circuit::Concat(_) => {
                         let new_ts = uniformize(&tensors_and_scales);
                         (
                             circ.eval_tensors(
                                 &new_ts.iter().map(|x| x.0.clone()).collect::<Vec<_>>(),
-                                &schedule.device_dtype,
                             )
                             .unwrap(),
                             new_ts[0].1,
@@ -702,7 +694,6 @@ pub fn evaluate_schedule_adjust_numerical_scale(
                         clamp(&(
                             circ.eval_tensors(
                                 &new_ts.iter().map(|x| x.0.clone()).collect::<Vec<_>>(),
-                                &schedule.device_dtype,
                             )
                             .unwrap(),
                             1.0,
@@ -714,7 +705,6 @@ pub fn evaluate_schedule_adjust_numerical_scale(
                                 .iter()
                                 .map(|x| x.0.clone())
                                 .collect::<Vec<_>>(),
-                            &schedule.device_dtype,
                         )
                         .unwrap(),
                         tensors_and_scales[0].1,
@@ -781,7 +771,6 @@ pub fn order_to_schedule(
             .map(|(h, n)| (h, ScheduleConstant::Circ(n)))
             .collect(),
     };
-
     let mut seen_dependencies: HashSet<usize> = HashSet::default();
     for ex in order.iter().rev() {
         for child in ex.non_free_children() {
@@ -796,11 +785,14 @@ pub fn order_to_schedule(
         }
         let next_id = circ_to_id.len();
         let our_id = *circ_to_id.entry(ex.info().hash).or_insert(next_id);
-        let node_here_symbol_children = ex
-            .map_non_free_children_unwrap(|child| {
-                child_from_key(circ_to_id[&child.info().hash], child.info().shape.clone()).rc()
-            })
-            .rc();
+        let node_here_symbol_children = (**ex.map_non_free_children_unwrap(|child| {
+            let mut result =
+                child_from_key(circ_to_id[&child.info().hash], child.info().shape.clone());
+            result.info_mut().device_dtype = child.info().device_dtype;
+            result.rc()
+        }))
+        .clone()
+        .rc();
         result
             .instructions
             .push(Instruction::Compute(our_id, node_here_symbol_children));
@@ -971,13 +963,7 @@ pub fn circuit_to_schedule(
         .iter()
         .map(|x| to_node[&dag.node_hashes[*x as usize]].clone())
         .collect();
-    if let Some(passed_dd) = context.settings.device_dtype {
-        circuit
-            .info()
-            .device_dtype
-            .combine(passed_dd.into())
-            .context("circuit device_dtype must match OptimizationSettings device_dtype")?;
-    }
+
     let (mut out, kept_keys) = order_to_schedule(
         &circuit_order,
         &to_node
@@ -989,10 +975,7 @@ pub fn circuit_to_schedule(
             .filter_map(|x| x.1.as_scalar().cloned())
             .collect(),
         HashSet::from_iter([circuit.info().hash]),
-        context
-            .settings
-            .device_dtype
-            .unwrap_or_else(|| get_compatible_dtype(&circuit)),
+        circuit.info().device_dtype.unwrap_or_defaults(),
     );
     out.output_circuit = Some((kept_keys[0], circuit));
     let out = replace_module_schedules(&out, context);
@@ -1000,10 +983,7 @@ pub fn circuit_to_schedule(
     Ok(out)
 }
 
-pub fn circuit_to_schedule_naive_toposort(
-    circuit: CircuitRc,
-    device_dtype: Option<TorchDeviceDtype>,
-) -> Result<Schedule> {
+pub fn circuit_to_schedule_naive_toposort(circuit: CircuitRc) -> Result<Schedule> {
     let circuit = substitute_all_modules(circuit);
     let toposorted = toposort_circuit(circuit.clone());
     let mut order = Vec::new();
@@ -1019,19 +999,13 @@ pub fn circuit_to_schedule_naive_toposort(
             order.push(c)
         }
     }
-    if let Some(device_dtype) = device_dtype {
-        circuit
-            .info()
-            .device_dtype
-            .combine(device_dtype.into())
-            .context("circuit device_dtype must match OptimizationSettings device_dtype")?;
-    }
+
     let (mut result, kept_keys) = order_to_schedule(
         &order,
         &constants,
         &scalars,
         HashSet::from_iter([circuit.info().hash]),
-        device_dtype.unwrap_or_else(|| get_compatible_dtype(&circuit)),
+        circuit.info().device_dtype.unwrap_or_defaults(),
     );
     result.output_circuit = Some((kept_keys[0], circuit));
     result.validate(true).expect("should be valid");
@@ -1041,7 +1015,7 @@ pub fn circuit_to_schedule_naive_toposort(
 #[pyfunction]
 pub fn scheduled_evaluate(circuit: CircuitRc, settings: OptimizationSettings) -> Result<Tensor> {
     let schedule = if settings.scheduling_naive {
-        circuit_to_schedule_naive_toposort(circuit, settings.device_dtype)
+        circuit_to_schedule_naive_toposort(circuit)
     } else {
         circuit_to_schedule(
             circuit.clone(),
