@@ -5,12 +5,8 @@ use std::{
 
 use anyhow::{bail, Result};
 use itertools::Itertools;
-use once_cell::unsync::Lazy;
 use rand::{rngs::SmallRng, Rng, SeedableRng};
-use rr_util::{
-    python_println, sv, timed, timed_value,
-    util::{is_unique, HashBytes},
-};
+use rr_util::{python_println, sv, timed, timed_value, util::HashBytes};
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
 use smallvec::SmallVec as Sv;
 
