@@ -12,7 +12,7 @@ use circuit_base::{
         Update as BaseUpdate, UpdatedIterativeMatcher as BaseUpdatedIterativeMatcher,
     },
     set_of_circuits::SetOfCircuitIdentities,
-    CircuitNode, CircuitRc,
+    CircuitNode, CircuitNodeUnion, CircuitRc, CircuitType,
 };
 use macro_rules_attribute::apply;
 use pyo3::{
@@ -24,7 +24,7 @@ use rr_util::{
     eq_by_big_hash::EqByBigHash,
     impl_both_by_big_hash,
     py_types::PyCallable,
-    python_error_exception, setup_callable, setup_wrap, simple_default, simple_from,
+    python_error_exception, setup_callable, simple_default, simple_from,
     tensor_util::Slice,
     util::{
         arc_ref_clone, arc_unwrap_or_clone, as_sorted, flip_op_result, split_first_take, transpose,
@@ -55,6 +55,7 @@ pub enum IterativeMatcherData {
     Term(bool),
     Restrict(RestrictIterativeMatcher),
     Children(ChildrenMatcher),
+    ModuleArg(ModuleArgMatcher),
     SpecCircuit(IterativeMatcherRc),
     NoModuleSpec(IterativeMatcherRc),
     Chains(BTreeSet<ChainItem>),
@@ -159,6 +160,7 @@ impl IterativeMatcherData {
             Self::Term(_) => uuid!("365bc6b4-cbc6-4d82-8e14-7505e1229eec"),
             Self::Restrict(_) => uuid!("bcc5ccaa-afbe-414d-88b4-b8eac8c93ece"),
             Self::Children(_) => uuid!("0081e9c9-7fdb-43e1-900e-f1adde28f23d"),
+            Self::ModuleArg(_) => uuid!("7a48bd8f-c028-4da8-9467-1277351aca35"),
             Self::SpecCircuit(_) => uuid!("b728a8c3-0493-4ee5-96c2-71e4900fc0f8"),
             Self::NoModuleSpec(_) => uuid!("b728a8c3-0493-4ee5-96c2-71e4900fc0f8"),
             Self::Chains(_) => uuid!("958d03ed-7a1a-4ea9-8dd4-d4a8a68feecb"),
@@ -200,6 +202,13 @@ impl IterativeMatcherData {
                     hasher.update(&num.to_le_bytes());
                 }
             }
+            Self::ModuleArg(ModuleArgMatcher {
+                module_matcher,
+                arg_sym_matcher,
+            }) => {
+                hasher.update(&module_matcher.hash);
+                hasher.update(&arg_sym_matcher.hash());
+            }
             Self::SpecCircuit(matcher) | Self::NoModuleSpec(matcher) => {
                 hasher.update(&matcher.hash);
             }
@@ -435,6 +444,76 @@ impl ChildrenMatcher {
     }
 }
 
+#[derive(Clone, Debug)]
+pub struct ModuleArgMatcher {
+    pub module_matcher: IterativeMatcherRc,
+    pub arg_sym_matcher: MatcherRc,
+}
+
+impl ModuleArgMatcher {
+    pub fn new(module_matcher: IterativeMatcherRc, arg_sym_matcher: MatcherRc) -> Result<Self> {
+        Ok(Self {
+            module_matcher,
+            arg_sym_matcher,
+        })
+    }
+
+    pub fn match_iterate(&self, circuit: CircuitRc) -> Result<IterateMatchResults> {
+        let IterateMatchResults { updated, found } =
+            self.module_matcher.match_iterate(circuit.clone())?;
+
+        let wrap = |updated_matcher| {
+            IterativeMatcherData::ModuleArg(Self {
+                module_matcher: updated_matcher,
+                arg_sym_matcher: self.arg_sym_matcher.clone(),
+            })
+            .into()
+        };
+
+        let wrapped_updated = if found {
+            let module =
+                circuit
+                    .as_module()
+                    .ok_or_else(|| IterativeMatcherError::ExpectedModule {
+                        circuit_type: circuit.type_tag(),
+                    })?;
+
+            let per_child = per_child(updated, self.module_matcher.clone(), circuit.num_children());
+            assert_eq!(per_child.len(), circuit.num_children());
+            Some(
+                per_child
+                    .into_iter()
+                    .enumerate()
+                    .map(|(i, x)| {
+                        let arg_matches = i > 0
+                            && self
+                                .arg_sym_matcher
+                                .call(module.spec.arg_specs[i - 1].symbol.crc())?;
+                        Ok(Some(
+                            IterativeMatcher::special_case_any(
+                                x.map(wrap)
+                                    .into_iter()
+                                    .chain(arg_matches.then(|| IterativeMatcher::term(true).into()))
+                                    .collect(),
+                            )
+                            .rc(),
+                        )
+                        .into())
+                    })
+                    .collect::<Result<Vec<Update>>>()?
+                    .into(),
+            )
+        } else {
+            map_updated(updated, wrap)
+        };
+
+        Ok(IterateMatchResults {
+            updated: wrapped_updated,
+            found: false,
+        })
+    }
+}
+
 impl circuit_base::opaque_iterative_matcher::HasTerm for IterativeMatcherRc {
     fn term() -> Self {
         IterativeMatcher::term(false).into()
@@ -576,8 +655,9 @@ impl IterativeMatcher {
             IterativeMatcherData::Match(m) => m.validate_matched(matched),
             IterativeMatcherData::Term(_) => Ok(()),
             IterativeMatcherData::Restrict(m) => m.iterative_matcher.validate_matched(matched),
-            IterativeMatcherData::Children(_) => Ok(()), /* not sure what else you could do here */
-            IterativeMatcherData::SpecCircuit(_) => Ok(()), /* not sure what else you could do here */
+            IterativeMatcherData::Children(_)
+            | IterativeMatcherData::ModuleArg(_)
+            | IterativeMatcherData::SpecCircuit(_) => Ok(()), /* not sure what else you could do here */
             IterativeMatcherData::NoModuleSpec(m) => m.validate_matched(matched),
             IterativeMatcherData::Chains(m) => {
                 for chain in m {
@@ -630,15 +710,18 @@ impl IterativeMatcher {
             IterativeMatcherData::Children(children_matcher) => {
                 children_matcher.match_iterate(circuit)?
             }
+            IterativeMatcherData::ModuleArg(module_arg_matcher) => {
+                module_arg_matcher.match_iterate(circuit)?
+            }
             IterativeMatcherData::SpecCircuit(module_matcher) => {
-                let is_module = circuit.is_module();
+                let tag = circuit.type_tag();
                 let IterateMatchResults { updated, found } =
                     module_matcher.match_iterate(circuit)?;
                 let updated_to_spec_circuit_matcher =
                     |iterative_matcher| IterativeMatcherData::SpecCircuit(iterative_matcher).into();
                 if found {
-                    if !is_module {
-                        bail!("not module!");
+                    if tag != CircuitType::Module {
+                        bail!(IterativeMatcherError::ExpectedModule { circuit_type: tag });
                     }
 
                     let per_child = per_child(updated, module_matcher.clone(), num_children);
@@ -1004,6 +1087,14 @@ impl IterativeMatcher {
         first_match.children_matcher(child_numbers)
     }
 
+    #[staticmethod]
+    pub fn new_module_arg_matcher(
+        module_matcher: IterativeMatcherRc,
+        arg_sym_matcher: MatcherRc,
+    ) -> Self {
+        module_matcher.module_arg_matcher(arg_sym_matcher)
+    }
+
     #[staticmethod]
     pub fn new_spec_circuit_matcher(module_matcher: IterativeMatcherRc) -> Self {
         module_matcher.spec_circuit_matcher()
@@ -1163,6 +1254,14 @@ impl IterativeMatcher {
         .into()
     }
 
+    pub fn module_arg_matcher(&self, arg_sym_matcher: MatcherRc) -> IterativeMatcher {
+        IterativeMatcherData::ModuleArg(ModuleArgMatcher {
+            module_matcher: self_.clone().rc(),
+            arg_sym_matcher,
+        })
+        .into()
+    }
+
     pub fn spec_circuit_matcher(&self) -> IterativeMatcher {
         IterativeMatcherData::SpecCircuit(self_.clone().rc()).into()
     }
@@ -1306,6 +1405,9 @@ pub enum IterativeMatcherError {
         child_numbers: BTreeSet<usize>,
         num_children: usize,
     },
+
+    #[error("matched circuit of type {circuit_type:?} is not a module ({e_name})")]
+    ExpectedModule { circuit_type: CircuitType },
 }
 
 #[pyfunction(
