@@ -2,12 +2,10 @@ use std::{
     collections::BTreeSet,
     fmt::{Display, Write},
     iter::{once, zip},
-    sync::Arc,
 };
 
 use anyhow::Result;
 use circuit_base::{
-    circuit_utils::toposort_circuit,
     clicolor,
     print::{color, last_child_arrows, CliColor, PrintOptions},
     CircuitNode, CircuitNodeUnion, CircuitRc,
@@ -19,7 +17,7 @@ use rr_util::{
     python_println,
     util::{indent, indent_butfirst},
 };
-use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
+use rustc_hash::FxHashSet as HashSet;
 
 use crate::{
     iterative_matcher::{per_child, ChainItem, IterativeMatcher},
@@ -122,6 +120,11 @@ impl Display for IterativeMatcher {
                     .map(|x| x.to_string())
                     .join(","),
             ),
+            IterativeMatcherData::ModuleArg(module_arg_matcher) => format!(
+                "ModuleArg({}, {})",
+                indented_if_multiline(&format!("{}", &module_arg_matcher.module_matcher.0)),
+                indented_if_multiline(&format!("{}", &module_arg_matcher.arg_sym_matcher.0)),
+            ),
             IterativeMatcherData::SpecCircuit(matcher) => format!(
                 "SpecCircuit({})",
                 indented_if_multiline(&format!("{}", &matcher.0))
