@@ -1,33 +1,34 @@
-use std::{
-    fmt::Debug,
-    iter::zip,
-    sync::{Arc, Mutex},
-};
+use std::{fmt::Debug, iter::zip, sync::Arc};
 
-use anyhow::{bail, Context, Error, Result};
-use cached::Cached;
+use anyhow::{bail, Error, Result};
 use circuit_base::{
     cumulant::{dim_permutation_for_circuits, partitions},
-    visit_circuit, Add, Array, CircResult, CircuitNode, CircuitRc, CircuitType, Cumulant,
+    evaluate,
+    generalfunction::multinomial,
+    visit_circuit, Add, Array, CircResult, Circuit, CircuitNode, CircuitRc, CircuitType, Cumulant,
     DiscreteVar, Einsum, GeneralFunction, Index, Scalar,
 };
-use circuit_rewrites::circuit_optimizer::{optimize_and_evaluate, OptimizationSettings};
+use circuit_rewrites::{
+    circuit_optimizer::OptimizationSettings,
+    generalfunction_rewrite::generalfunction_gen_index_const_to_index,
+};
 use itertools::Itertools;
 use macro_rules_attribute::apply;
 use pyo3::{exceptions::PyValueError, prelude::*};
 use rr_util::{
     cached_method,
     caching::FastUnboundedCache,
+    eq_by_big_hash::EqByBigHash,
     fn_struct,
-    py_types::{scalar_to_tensor, PyShape, Tensor, PY_UTILS},
-    python_error_exception, sv,
+    py_types::{i64_to_tensor, random_torch_i64, PyShape},
+    python_error_exception,
     tensor_util::{
-        Shape, Slice, TensorAxisIndex, TensorIndex, TorchDeviceDtype, TorchDeviceDtypeOp,
+        Shape, Slice, TensorAxisIndex, TensorIndex, TorchDevice, TorchDeviceDtype, TorchDtype,
     },
     tu8v,
-    util::{flip_op_result, HashBytes},
+    util::HashBytes,
 };
-use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
+use rustc_hash::FxHashSet as HashSet;
 use smallvec::SmallVec as Sv;
 use thiserror::Error;
 
@@ -36,7 +37,15 @@ use crate::{
     Matcher, Transform, TransformRc,
 };
 
-fn_struct!(pub Seeder:Fn(circuit:CircuitRc)->Option<i32>);
+fn_struct!(pub Seeder:Fn(circuit:CircuitRc)->i64);
+
+#[pyfunction]
+pub fn default_hash_seeder(base_seed: Option<i64>) -> Seeder {
+    let base_seed = base_seed.unwrap_or_else(|| random_torch_i64());
+    Seeder::Dyn(SeederDynStruct(Arc::new(move |circ| {
+        Ok(circ.first_i64() ^ base_seed)
+    })))
+}
 
 fn count_elements(s: &Shape) -> usize {
     s.iter().product()
@@ -56,7 +65,7 @@ pub fn default_var_matcher() -> IterativeMatcherRc {
 
 pub trait SampleSpecTrait {
     /// Like the python method, but modify inplace (via mutex or other interior mutability)
-    fn sample_var(&self, d: &DiscreteVar, device_dtype: TorchDeviceDtypeOp) -> Result<CircuitRc>;
+    fn sample_var(&self, d: &DiscreteVar) -> Result<CircuitRc>;
     fn get_empirical_weights(&self) -> CircuitRc;
     fn get_overall_weight(&self) -> CircuitRc {
         Scalar::nrc(
@@ -66,27 +75,23 @@ pub trait SampleSpecTrait {
         )
     }
     fn get_sample_shape(&self) -> Shape;
-    fn get_transform(self, device_dtype: TorchDeviceDtypeOp) -> TransformRc
+    fn get_transform(self) -> TransformRc
     where
         Self: Sized + Sync + Send + 'static,
     {
-        Transform::new_func_err(move |circ| {
-            self.sample_var(circ.as_discrete_var_unwrap(), device_dtype.clone())
-        })
-        .rc()
+        Transform::new_func_err(move |circ| self.sample_var(circ.as_discrete_var_unwrap())).rc()
     }
     fn get_sample_expander(
         self,
         var_matcher: IterativeMatcherRc,
         default_fancy_validate: bool,
         suffix: Option<String>,
-        device_dtype: TorchDeviceDtypeOp,
     ) -> Expander
     where
         Self: Sized + Sync + Send + 'static,
     {
         Expander::new(
-            vec![(var_matcher, self.get_transform(device_dtype))],
+            vec![(var_matcher, self.get_transform())],
             true,
             default_fancy_validate,
             suffix,
@@ -108,9 +113,9 @@ pub struct RandomSampleSpec {
     #[pyo3(get, set)]
     probs_and_group_evaluation_settings: OptimizationSettings,
     #[pyo3(get, set)]
-    seeder: Option<Seeder>,
-    // kinda dumb that we use a mutex here. Certainly overkill in a particular sense...
-    indices_cache: Arc<Mutex<FastUnboundedCache<HashBytes, Tensor>>>,
+    seeder: Seeder,
+    #[pyo3(get, set)]
+    simplify: bool,
 }
 
 macro_rules! py_forward_sample_spec {
@@ -118,13 +123,8 @@ macro_rules! py_forward_sample_spec {
         #[pymethods]
         impl $type {
             #[pyo3(name = "sample_var")]
-            #[args(device_dtype = "TorchDeviceDtypeOp::NONE")]
-            fn sample_var_py(
-                &self,
-                d: &DiscreteVar,
-                device_dtype: TorchDeviceDtypeOp,
-            ) -> Result<CircuitRc> {
-                self.sample_var(d, device_dtype)
+            fn sample_var_py(&self, d: &DiscreteVar) -> Result<CircuitRc> {
+                self.sample_var(d)
             }
             #[pyo3(name = "get_empirical_weights")]
             fn get_empirical_weights_py(&self) -> CircuitRc {
@@ -139,30 +139,23 @@ macro_rules! py_forward_sample_spec {
                 PyShape(self.get_sample_shape())
             }
             #[pyo3(name = "get_transform")]
-            #[args(device_dtype = "TorchDeviceDtypeOp::NONE")]
-            fn get_transform_py(&self, device_dtype: TorchDeviceDtypeOp) -> TransformRc {
-                self.clone().get_transform(device_dtype)
+            fn get_transform_py(&self) -> TransformRc {
+                self.clone().get_transform()
             }
             #[pyo3(name = "get_sample_expander")]
             #[args(
                 var_matcher = "default_var_matcher()",
                 default_fancy_validate = "false",
-                suffix = "\"sample\".to_owned()",
-                device_dtype = "TorchDeviceDtypeOp::NONE"
+                suffix = "\"sample\".to_owned()"
             )]
             fn get_sample_expander_py(
                 &self,
                 var_matcher: IterativeMatcherRc,
                 default_fancy_validate: bool,
                 suffix: Option<String>,
-                device_dtype: TorchDeviceDtypeOp,
             ) -> Expander {
-                self.clone().get_sample_expander(
-                    var_matcher,
-                    default_fancy_validate,
-                    suffix,
-                    device_dtype,
-                )
+                self.clone()
+                    .get_sample_expander(var_matcher, default_fancy_validate, suffix)
             }
         }
 
@@ -193,71 +186,60 @@ impl RandomSampleSpec {
         shape = "RandomSampleSpec::default().shape",
         replace = "RandomSampleSpec::default().replace",
         probs_and_group_evaluation_settings = "RandomSampleSpec::default().probs_and_group_evaluation_settings",
+        simplify = "RandomSampleSpec::default().simplify",
         seeder = "None"
     )]
     fn new(
         shape: Shape,
         replace: bool,
         probs_and_group_evaluation_settings: OptimizationSettings,
+        simplify: bool,
         seeder: Option<Seeder>,
     ) -> PyClassInitializer<Self> {
+        let seeder = seeder.unwrap_or_else(|| default_hash_seeder(None));
         Self {
             shape,
             replace,
+            simplify,
             probs_and_group_evaluation_settings,
             seeder,
             ..Default::default()
         }
         .into_init()
     }
-
-    pub fn indices_cache(&self) -> HashMap<HashBytes, Tensor> {
-        self.indices_cache.lock().unwrap().0.clone()
-    }
 }
 
 impl RandomSampleSpec {
-    fn sample_impl(
-        &self,
-        probs_and_group: CircuitRc,
-        device_dtype: TorchDeviceDtype,
-    ) -> Result<Tensor> {
-        let key = probs_and_group.info().hash;
-        if let Some(out) = self.indices_cache.lock().unwrap().cache_get(&key) {
-            return Ok(out.clone());
-        }
+    fn sample_impl(&self, probs_and_group: CircuitRc) -> Result<CircuitRc> {
+        let seed = self.seeder.call(probs_and_group.clone())?;
+        let seed_array = Array::nrc(
+            i64_to_tensor(
+                seed,
+                Shape::new(),
+                TorchDeviceDtype {
+                    device: TorchDevice::Cpu,
+                    dtype: TorchDtype::int64,
+                },
+            )
+            .unwrap(),
+            Some(format!("seed_{seed}").into()),
+        );
 
-        let prob_tensor = optimize_and_evaluate(
-            Add::nrc(
-                vec![
-                    probs_and_group.clone(),
-                    Array::nrc(
-                        scalar_to_tensor(0., sv![], device_dtype)
-                            .context("failed to create scalar tensor???")?,
-                        None,
-                    ),
-                ],
-                None,
-            ),
-            self.probs_and_group_evaluation_settings,
+        let name = probs_and_group
+            .info()
+            .name
+            .map(|n| format!("{n} sampled_idxs").into());
+        let indices = multinomial(
+            probs_and_group,
+            seed_array,
+            self.shape.clone(),
+            self.replace,
+            name,
         )
-        .context("failed to evaluate prob group")?;
+        .unwrap() // is this unwrap actually always fine?
+        .rc();
 
-        let seed = flip_op_result(
-            self.seeder
-                .clone()
-                .map(|seed_fn| seed_fn.call(probs_and_group)),
-        )?
-        .flatten();
-
-        let out = sample_indices(prob_tensor, self.shape.clone(), self.replace, seed)
-            .context("failed to sample indices for discrete var")?;
-
-        self.indices_cache
-            .lock()
-            .unwrap()
-            .cache_set(key, out.clone());
-        Ok(out)
+        Ok(indices)
     }
 }
 
@@ -266,70 +248,45 @@ impl Default for RandomSampleSpec {
         RandomSampleSpec {
             shape: Sv::new(),
             replace: true,
+            simplify: true,
             probs_and_group_evaluation_settings: Default::default(),
-            seeder: None,
-            indices_cache: Arc::new(Mutex::new(FastUnboundedCache::default())),
+            seeder: default_hash_seeder(Some(0)), // NOTE: this is deterministic!
         }
     }
 }
 
-fn combine_dtype_device_for_sample(
-    var: &DiscreteVar,
-    device_dtype: TorchDeviceDtypeOp,
-) -> Result<TorchDeviceDtype> {
-    let var_probs_dtype_device = TorchDeviceDtypeOp {
-        device: var.info().device_dtype.device,
-        dtype: var.probs_and_group.info().device_dtype.dtype,
-    };
-    device_dtype
-        .clone()
-        .combine(var_probs_dtype_device)
-        .with_context(|| {
-            format!(
-                concat!(
-                    "sampled var device_dtype failed to combine with passed device_dtype",
-                    "var probs dtype_device={:?} passed device_dtype={:?}"
-                ),
-                var_probs_dtype_device, device_dtype
-            )
-        })
-        .map(|x| x.unwrap_or_defaults())
-}
-
-fn sample_indices(
-    input: Tensor,
-    shape: Shape,
-    replacement: bool,
-    seed: Option<i32>,
-) -> Result<Tensor, PyErr> {
-    let indices = Python::with_gil(|py| {
-        PY_UTILS
-            .random_indices
-            .call(py, (input, shape.clone(), replacement, seed), None)?
-            .extract::<Tensor>(py)
-    })?;
-    assert_eq!(indices.shape(), &shape);
-    Ok(indices)
-}
 impl SampleSpecTrait for RandomSampleSpec {
-    fn sample_var(&self, var: &DiscreteVar, device_dtype: TorchDeviceDtypeOp) -> Result<CircuitRc> {
-        let device_dtype = combine_dtype_device_for_sample(var, device_dtype)?;
-        let indices = self
-            .sample_impl(var.probs_and_group.clone(), device_dtype)?
-            .flatten() // flatten here, unflatten via rearrange
-            .try_into()
-            .unwrap();
-
-        // TODO: use fancier approach when index supports that
-        let res = Index::nrc(
+    fn sample_var(&self, var: &DiscreteVar) -> Result<CircuitRc> {
+        let index = self.sample_impl(var.probs_and_group.clone())?;
+        let index = if self.simplify
+            && index.info().is_explicitly_computable()
+            && var.probs_and_group.is_tag()
+            && let Circuit::Tag(c) = &**var.probs_and_group
+            && c.node.is_leaf_constant()
+        {
+            evaluate(index.clone())
+                .and_then(|x| Ok(Array::try_new(x, index.info().name)?.rc()))
+                .unwrap_or(index)
+        } else {
+            index
+        };
+        let batch_x = true;
+        let batch_index = false;
+        let res = GeneralFunction::gen_index(
             var.values.clone(),
-            TensorIndex::new_single(TensorAxisIndex::Tensor(indices), 0, 1),
-            None,
+            index,
+            -(var.values.ndim() as i64),
+            batch_x,
+            batch_index,
+            true,
+            var.info().name.map(|n| format!("{n} sampled").into()),
         )
-        .unflatten_axis(0, self.shape.clone(), None)
-        .unwrap()
-        .rc();
-        Ok(res)
+        .unwrap();
+        Ok(if self.simplify {
+            generalfunction_gen_index_const_to_index(&res).unwrap_or(res.rc())
+        } else {
+            res.rc()
+        })
     }
     fn get_empirical_weights(&self) -> CircuitRc {
         Scalar::new(
@@ -437,7 +394,7 @@ impl RunDiscreteVarAllSpec {
 }
 
 impl SampleSpecTrait for RunDiscreteVarAllSpec {
-    fn sample_var(&self, var: &DiscreteVar, _device_dtype: TorchDeviceDtypeOp) -> CircResult {
+    fn sample_var(&self, var: &DiscreteVar) -> CircResult {
         let this_group_i = match self
             .groups
             .iter()
@@ -538,8 +495,8 @@ impl SampleSpec {
 }
 
 impl SampleSpecTrait for SampleSpec {
-    fn sample_var(&self, d: &DiscreteVar, device_dtype: TorchDeviceDtypeOp) -> CircResult {
-        self.as_trait_obj().sample_var(d, device_dtype)
+    fn sample_var(&self, d: &DiscreteVar) -> CircResult {
+        self.as_trait_obj().sample_var(d)
     }
     fn get_empirical_weights(&self) -> CircuitRc {
         self.as_trait_obj().get_empirical_weights()
@@ -576,21 +533,19 @@ impl Sampler {
         var_matcher = "default_var_matcher()",
         cumulant_matcher = "Matcher::types(vec![CircuitType::Cumulant]).into()",
         suffix = "\"sample\".to_owned()",
-        device_dtype = "TorchDeviceDtypeOp::NONE",
-        run_on_sampled = "Transform::ident().rc()"
+        run_on_sampled = "Transform::ident().rc()",
+        simplify_Sampled = "true"
     )]
     fn new(
         sample_spec: SampleSpec,
         var_matcher: IterativeMatcherRc,
         cumulant_matcher: IterativeMatcherRc,
         suffix: Option<String>,
-        device_dtype: TorchDeviceDtypeOp,
         run_on_sampled: TransformRc,
     ) -> Self {
-        let expander =
-            sample_spec
-                .clone()
-                .get_sample_expander(var_matcher, false, suffix, device_dtype);
+        let expander = sample_spec
+            .clone()
+            .get_sample_expander(var_matcher, false, suffix);
         Self {
             expander,
             cumulant_matcher,
