@@ -12,6 +12,7 @@ use itertools::Itertools;
 use macro_rules_attribute::apply;
 use pyo3::{exceptions::PyValueError, prelude::*};
 use rr_util::{
+    name::Name,
     python_error_exception,
     rearrange_spec::{OpSize, RInts, RearrangeSpec},
     sv,
@@ -32,14 +33,14 @@ pub fn rewrite_cum_to_circuit_of_cum(
     on_sub_cumulant_fn: Option<PyObject>,
 ) -> CircResult {
     if cumulant.nodes.len() == 0 {
-        return Ok(Scalar::nrc(1., sv![], cumulant.name_cloned()));
+        return Ok(Scalar::nrc(1., sv![], cumulant.info().name));
     }
 
     let on_sub_cumulant = get_on_sub_cumulant(&namer, &on_sub_cumulant_fn);
 
     let (node_position, start_axis) = find_position_and_axis_of_start(&cumulant, &node)?;
 
-    let original_cumulant_name = cumulant.name_cloned();
+    let original_cumulant_name = cumulant.info().name;
 
     let r = match &**node {
         Circuit::Rearrange(n) => {
@@ -141,9 +142,9 @@ pub fn rewrite_cum_to_circuit_of_cum(
         }
         Circuit::Array(_) | Circuit::Scalar(_) | Circuit::Cumulant(_) => {
             if cumulant.nodes.len() == 1 {
-                node.rename(cumulant.name_cloned()).rc()
+                node.rename(cumulant.info().name).rc()
             } else {
-                Scalar::nrc(0., cumulant.shape().clone(), cumulant.name_cloned())
+                Scalar::nrc(0., cumulant.shape().clone(), cumulant.info().name)
             }
         }
         Circuit::GeneralFunction(_) | Circuit::Symbol(_) => {
@@ -155,7 +156,7 @@ pub fn rewrite_cum_to_circuit_of_cum(
 }
 
 fn replace_node(cumulant: Cumulant, node: CircuitRc, position: usize) -> Cumulant {
-    let new_name = cumulant.name_cloned();
+    let new_name = cumulant.info().name;
     let mut new_nodes = cumulant.nodes;
     new_nodes[position] = node;
     Cumulant::new(new_nodes, new_name)
@@ -238,7 +239,7 @@ fn cum_of_einsum_to_einsum_of_cum_raw(
                 let operated_on_prod = Einsum::try_new(
                     vec![(prod.rc(), einsum.input_axes().flatten().cloned().collect())],
                     out_nums.clone().into(),
-                    Some("perm".to_owned()), /* name=Cumulant.get_name(*new_cums, getter=namer) + " perm",  # naming to ensure no duplicates */
+                    Some("perm".into()), /* name=Cumulant.get_name(*new_cums, getter=namer) + " perm",  # naming to ensure no duplicates */
                 )?; // TODO: better name
 
                 to_sum.push(
@@ -252,10 +253,10 @@ fn cum_of_einsum_to_einsum_of_cum_raw(
         Ok(Scalar::nrc(
             0.,
             cumulant.shape().clone(),
-            cumulant.name_cloned(),
+            cumulant.info().name,
         ))
     } else {
-        Ok(Add::nrc(to_sum, cumulant.name_cloned()))
+        Ok(Add::nrc(to_sum, cumulant.info().name))
     }
 }
 
@@ -323,11 +324,13 @@ fn get_on_sub_cumulant<'a>(
     Box::new(move |c: Cumulant| -> CircResult {
         let renamed_cumulant = match namer {
             Some(namer_fn) => {
-                let cumulant_new_name = Python::with_gil(|py| -> Result<Option<String>> {
-                    Ok(namer_fn
-                        .call(py, (c.clone(),), None)?
-                        .extract::<Option<String>>(py)?)
-                })?;
+                let cumulant_new_name: Option<Name> =
+                    Python::with_gil(|py| -> Result<Option<Name>> {
+                        Ok(namer_fn
+                            .call(py, (c.clone(),), None)?
+                            .extract::<Option<String>>(py)?
+                            .map(|z| z.into()))
+                    })?;
                 c.rename(cumulant_new_name)
             }
             None => c, // Keep the name generated by autonaming
