@@ -17,7 +17,7 @@ use circuit_rewrites::{
     },
 };
 use get_update_node::{AnyFound, IterativeMatcher, IterativeMatcherRc, MatcherData};
-use itertools::multiunzip;
+use itertools::{izip, multiunzip};
 use macro_rules_attribute::apply;
 use pyo3::{
     exceptions::{PyRuntimeError, PyValueError},
@@ -810,16 +810,21 @@ impl ModulePusher {
                     .zip(node_get_skip)
                     // don't use rec to avoid adding nodes to out_set
                     .map(|(c, (get, skip))| {
-                        let ndim = c.ndim();
                         let out = unwrap!(self_.push_down_modules_rec(
-                            c,
+                            c.clone(),
                             get,
                             skip,
                             mode,
                             &Default::default(), // no items (only difference from recursive call above)
                             extra_replacements,
                         )?.0, CircOrGet::Circ);
-                        assert_eq!(out.ndim(), ndim);
+                        assert_eq!(
+                            out.ndim(),
+                            c.ndim(),
+                            "expected to have same ndim: \n{}\n{}",
+                            out.repru(),
+                            c.repru()
+                        );
                         Ok(out)
                     })
                     .collect::<Result<Vec<_>>>()?;
@@ -869,14 +874,14 @@ impl ModulePusher {
                 }
             }
 
-            let new_children = circuit
-                .children()
-                .into_iter()
-                .zip(new_get.clone())
-                .zip(new_skip_module.clone())
-                .zip(extra_replacements.per_child(&circuit))
-                .map(|(((c, get), skip), rep)| rec(self_, c, get, skip, items, &rep).map(|x| x.0))
-                .collect::<Result<Vec<_>>>()?;
+            let new_children = izip!(
+                circuit.children(),
+                new_get.clone(),
+                new_skip_module.clone(),
+                extra_replacements.per_child(&circuit)
+            )
+            .map(|(c, get, skip, rep)| rec(self_, c, get, skip, items, &rep).map(|x| x.0))
+            .collect::<Result<Vec<_>>>()?;
 
             if is_get {
                 return Ok((CircOrGet::Get(out_set), extra_default));
