@@ -14,7 +14,7 @@ use rr_util::{
     python_error_exception,
     rearrange_spec::{check_permutation, PermError},
     union_find::UnionFind,
-    util::{is_unique, transpose, AsOp, AxisInt, EinsumAxes, Multizip},
+    util::{is_unique, transpose, AsOp, EinsumAxes, Multizip},
 };
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
 use thiserror::Error;
@@ -414,7 +414,7 @@ impl NestFullInfo for NestEinsumsFullInfo {
         let (union_find, int_map) = extra;
         assert_eq!(circ.is_einsum(), int_map.is_some());
         if let Some(int_map) = int_map {
-            let get_new_nums = |x: u8| union_find.find(int_map[&x] as usize) as u8;
+            let get_new_nums = |x: u8| union_find.find_(int_map[&x] as usize) as u8;
             x.map_out_axes(get_new_nums) // quadratic in tree depth fwiw
         } else {
             x
@@ -1271,7 +1271,7 @@ pub fn register(py: Python<'_>, m: &PyModule) -> PyResult<()> {
 pub enum NestError {
     #[error("count_rest={count_rest} ({e_name})")]
     MultipleRest { count_rest: usize },
-    #[error("matcher={matcher:?} {}={other_matcher:?}\n{ints:?} intersects with {other_ints:?}\nintersection={intersection:?}\nintersection_circs={intersection_circs:?} ({e_name})", 
+    #[error("matcher={matcher:?} {}={other_matcher:?}\n{ints:?} intersects with {other_ints:?}\nintersection={intersection:?}\nintersection_circs={intersection_circs:?} ({e_name})",
         "overlaps with other_matcher")]
     MatchersOverlap {
         matcher: IterativeMatcherRc,
@@ -1282,8 +1282,8 @@ pub enum NestError {
         intersection_circs: Vec<CircuitRc>,
     },
     #[error(
-        "matcher={matcher:?}, matcher_ints={matcher_ints:?} {}{explicit_ints:?}\nintersection={intersection:?}\n{}{intersection_circs:?} ({e_name})", 
-        "intersects with explicit_ints=", 
+        "matcher={matcher:?}, matcher_ints={matcher_ints:?} {}{explicit_ints:?}\nintersection={intersection:?}\n{}{intersection_circs:?} ({e_name})",
+        "intersects with explicit_ints=",
         "intersection_circs=")]
     MatcherOverlapsWithExplicitInts {
         matcher: IterativeMatcherRc,
