@@ -2,7 +2,7 @@
 import functools
 import itertools
 import math
-from typing import List, Optional, Tuple
+from typing import List, Optional, Sequence, Tuple, Union
 
 import torch
 import torch.nn.functional
@@ -75,6 +75,7 @@ generalfunctions = {
     "sin": torch.sin,
     "cos": torch.cos,
     "gelu": torch.nn.functional.gelu,
+    "gelu_new": lambda x: 0.5 * x * (1.0 + torch.tanh(math.sqrt(2.0 / math.pi) * (x + 0.044715 * torch.pow(x, 3.0)))),
     "relu": torch.relu,
     "step": lambda x: torch.where(
         x > 0.0, torch.ones((), dtype=x.dtype, device=x.device), torch.zeros((), dtype=x.dtype, device=x.device)
@@ -115,29 +116,45 @@ def check_ints(x: torch.Tensor):
 
 
 def gen_index_function(
-    x: torch.Tensor, index: torch.Tensor, index_dim: int, use_index_as_prefix: bool, check_index_ints: bool
+    x: torch.Tensor, index: torch.Tensor, index_dim: int, batch_x: bool, batch_index: bool, check_index_ints: bool
 ):
-    # copy of  WildIndex from computational_node.py (maybe dedup?)
+    # copy of  WildIndex from computational_node.py
     if check_index_ints:
         check_ints(index)
+    use_index_as_prefix = batch_x and batch_index
     if use_index_as_prefix:
-        prefix_len = index.ndim
-        assert x.shape[:prefix_len] == index.shape
-        suffix_len = x.ndim - prefix_len
-        moved_x = x.movedim(check_canon_idx(index_dim, suffix_len) + prefix_len, 0)
-        if moved_x.ndim == 1:
-            assert index.ndim == 0
-            return moved_x[index.long()]
-        else:
-            # end_dim is inclusive, so this is right
-            flattened_x = moved_x.flatten(start_dim=1, end_dim=len(index.shape))
-        flattened_index = index.flatten().long()
-        new_x = flattened_x[flattened_index, torch.arange(index.numel())]
-        return new_x.reshape(moved_x.shape[1:])
-
-    else:
+        return explicit_gen_index_function(x, index, index_dim, x.ndim - index.ndim, check_index_ints=check_index_ints)
+    elif batch_index:
         moved_out = torch.index_select(x, index_dim, index.long().flatten()).moveaxis(index_dim, 0)
         return moved_out.reshape((*index.shape, *moved_out.shape[1:]))
+    else:
+        canon_index_dim = check_canon_idx(index_dim, x.ndim)
+        out = torch.index_select(x, canon_index_dim, index.long().flatten())
+        return out.reshape((*out.shape[:canon_index_dim], *index.shape, *out.shape[canon_index_dim + 1 :]))
+
+
+def explicit_gen_index_function(
+    x: torch.Tensor, index: torch.Tensor, index_dim: int, x_non_batch_dims: int, check_index_ints: bool
+):
+    if check_index_ints:
+        check_ints(index)
+
+    batch_len = x.ndim - x_non_batch_dims
+    batch_shape = x.shape[:batch_len]
+    assert batch_shape == index.shape[:batch_len]
+    # x_len = x.ndim - prefix_len
+    moved_x = x.movedim(check_canon_idx(index_dim, x_non_batch_dims) + batch_len, 0)
+    if moved_x.ndim == 1:
+        return moved_x[index.long()]
+
+    # end_dim is inclusive, so this is right
+    flattened_x = moved_x.flatten(start_dim=1, end_dim=batch_len)
+    flattened_index = index.flatten(start_dim=0, end_dim=batch_len - 1).long()
+
+    together_arange = torch.arange(math.prod(batch_shape))
+    pad_arange = together_arange[(slice(None), *([None] * (index.ndim - batch_len)))]
+    new_x = flattened_x[flattened_index, pad_arange]
+    return new_x.reshape((*index.shape, *moved_x.shape[1 + batch_len :]))
 
 
 assert_tensors_close = torch.testing.assert_close
@@ -174,19 +191,23 @@ def tensor_from_bytes(device_dtype, shape, bytes, count):
 einsum = lambda tensors_and_axes, out_axes: torch.einsum(*itertools.chain(*tensors_and_axes), out_axes)
 
 
-def random_indices(input: torch.Tensor, shape, replacement: bool, seed: Optional[int] = None) -> torch.Tensor:
+def random_indices(
+    probs: torch.Tensor, shape: Sequence[int], replacement: bool, seed: Optional[Union[int, torch.Tensor]] = None
+) -> torch.Tensor:
     shape = tuple(shape)
 
+    assert probs.ndim >= 1
+
     generator: Optional[torch.Generator] = (
-        torch.Generator(device=input.device).manual_seed(seed) if seed is not None else None
+        torch.Generator(device=probs.device).manual_seed(int(seed)) if seed is not None else None
     )
 
     return torch.multinomial(
-        input,
+        probs.reshape(math.prod(probs.shape[:-1]), probs.shape[-1]),
         num_samples=math.prod(shape),
         replacement=replacement,
         generator=generator,
-    ).reshape(shape)
+    ).reshape((*probs.shape[:-1], *shape))
 
 
 # we use different axis layout than pytorch,
@@ -232,3 +253,7 @@ def conv(dim, input, filter, stride, padding):
 
 class OptimizingSymbolicSizeWarning(UserWarning):
     ...
+
+
+def random_i64():
+    return int(torch.randint(-(2 ** 63), 2 ** 63 - 1, size=(), dtype=torch.long))
