@@ -271,7 +271,8 @@ impl RearrangeSpec {
             output_ints,
             int_sizes,
         };
-        out.check_valid()?;
+        out.check_valid()
+            .context(format!("Invalid rearrange spec: {:?}", out))?;
         Ok(out)
     }
 
