@@ -10,7 +10,7 @@ use z3::{self, ast, ast::Ast};
 
 use crate::{
     python_error_exception,
-    tensor_util::{parse_numeric, ParseError, UINT_WITH_UNDERSCORE},
+    tensor_util::{parse_numeric, ParseError, Shape, UINT_WITH_UNDERSCORE},
 };
 
 const PRIME_BYTES: &[u8] = include_bytes!(concat!(env!("CARGO_MANIFEST_DIR"), "/primes.bin"));
@@ -429,3 +429,9 @@ pub enum SymbolicSizeSetError {
         constraints: BTreeSet<SymbolicSizeConstraint>,
     },
 }
+
+pub fn shape_first_symbolic_dim(shape: &Shape) -> Option<usize> {
+    shape
+        .iter()
+        .position(|x| SymbolicSizeProduct::has_symbolic(*x))
+}
