@@ -216,30 +216,39 @@ pub fn ensure_all_tensors_local(hashes: Vec<String>) -> Result<()> {
 
 #[pyfunction(parallelism = "15")]
 pub fn sync_all_unsynced_tensors(parallelism: usize) -> Result<()> {
-    let list_dir = get_tensor_cache_dir() + "/.unsynced_list";
-    if !fs::try_exists(&list_dir)? {
-        println!("didnt find any usynced list (~/tensors_by_hash_cache/.unsynced_list), returning");
+    let unsynced_list_path = get_tensor_cache_dir() + "/.unsynced_list";
+    if !fs::try_exists(&unsynced_list_path)? {
+        println!(
+            "didnt find any unsynced list (~/tensors_by_hash_cache/.unsynced_list), returning"
+        );
         return Ok(());
     }
     println!("starting to read list");
-    let all_unsynced = fs::read_to_string(&list_dir)?;
+    let all_unsynced = fs::read_to_string(&unsynced_list_path)?;
     println!("read list {}", all_unsynced.len());
     let lines: Vec<String> = all_unsynced
         .lines()
         .filter(|x| !x.is_empty())
         .map(|x| x.to_owned())
         .collect();
+
+    sync_specific_tensors(lines, parallelism)
+}
+
+#[pyfunction(parallelism = "15")]
+pub fn sync_specific_tensors(tensor_hash_prefixes: Vec<String>, parallelism: usize) -> Result<()> {
     // we can have more threads than cores bc we're not actually doing computation on threads
     // ideally would be using async
     let n_workers = parallelism;
     let pool = ThreadPool::new(n_workers);
     let (tx, rx) = channel();
-    for line in &lines {
-        let line = line.clone();
+    for hash_prefix_str in &tensor_hash_prefixes {
+        let hash_prefix_str = hash_prefix_str.clone();
         let tx = tx.clone();
         pool.execute(move || {
             tx.send((move || {
-                let local_filename = get_filename_dir_tree(&get_tensor_cache_dir(), &line)?.0;
+                let local_filename =
+                    get_filename_dir_tree(&get_tensor_cache_dir(), &hash_prefix_str)?.0;
                 let remote_filename =
                     local_filename.replace(&get_tensor_cache_dir(), &get_rrfs_tensor_db_dir());
                 let remote_filename_dir: Vec<_> = remote_filename.split("/").collect();
@@ -249,14 +258,35 @@ pub fn sync_all_unsynced_tensors(parallelism: usize) -> Result<()> {
                 fs::create_dir_all(&remote_filename_dir).context("failed to create dir in tree")?;
                 copy_without_setting_permissions(&local_filename, &remote_filename)
                     .context("failed to copy")?;
-                eprintln!("wrote {}", line);
+                eprintln!("wrote {}", hash_prefix_str);
                 Ok::<(), anyhow::Error>(())
             })())
             .unwrap()
         });
     }
-    rx.iter().take(lines.len()).collect::<Result<Vec<()>>>()?;
-    fs::write(&list_dir, &fs::read(&list_dir)?[all_unsynced.len()..]).context("write failed")
+    rx.iter()
+        .take(tensor_hash_prefixes.len())
+        .collect::<Result<Vec<()>>>()?;
+
+    let unsynced_list_path = get_tensor_cache_dir() + "/.unsynced_list";
+    if !fs::try_exists(&unsynced_list_path)? {
+        println!(
+            "didnt find any unsynced list (~/tensors_by_hash_cache/.unsynced_list), returning"
+        );
+        return Ok(());
+    }
+
+    let all_unsynced = fs::read_to_string(&unsynced_list_path)?;
+    println!("read list {}", all_unsynced.len());
+    let still_unsynced_lines: Vec<String> = all_unsynced
+        .lines()
+        // removing all with prefix is fine because we check prefix only matched one above
+        // (Up to race condition on matching prefix - we probably should have some sort of lock etc!)
+        .filter(|x| !x.is_empty() && !tensor_hash_prefixes.iter().any(|p| x.starts_with(p))) // note any is inefficient, but who cares...
+        .map(|x| x.to_owned() + "\n")
+        .collect();
+
+    fs::write(&unsynced_list_path, still_unsynced_lines.join("")).context("write failed")
 }
 
 #[pyfunction]
