@@ -983,6 +983,17 @@ impl TorchDeviceDtypeOp {
         Self { device, dtype }
     }
 
+    pub fn only_keep_floating(&self) -> Self {
+        Self {
+            device: self.device,
+            dtype: if self.dtype.map(|x| x.is_floating_point()).unwrap_or(false) {
+                self.dtype
+            } else {
+                None
+            },
+        }
+    }
+
     pub fn __repr__(&self) -> String {
         format!(
             "TorchDeviceDtypeOp(device={}, dtype={})",
@@ -1084,6 +1095,21 @@ impl TorchDtype {
     pub fn get_torch(self) -> PyObject {
         PY_UTILS.dtype_convert[&self].clone()
     }
+
+    pub fn is_floating_point(self) -> bool {
+        matches!(
+            self,
+            Self::float16 | Self::bfloat16 | Self::float32 | Self::float64
+        )
+    }
+
+    // pub fn is_integral(self) -> bool {
+    //     // I assume this shouldn't include bool? idk though
+    //     matches!(
+    //         self,
+    //         Self::int8 | Self::int16 | Self::int32 | Self::int64 | Self::uint8
+    //     )
+    // }
 }
 
 string_enum! {
@@ -1111,6 +1137,31 @@ string_enum! {
     )
 }
 
+impl TorchDevice {
+    pub fn is_cuda(&self) -> bool {
+        matches!(
+            self,
+            Self::Cuda
+                | Self::Cuda0
+                | Self::Cuda1
+                | Self::Cuda2
+                | Self::Cuda3
+                | Self::Cuda4
+                | Self::Cuda5
+                | Self::Cuda6
+                | Self::Cuda7
+                | Self::Cuda8
+                | Self::Cuda9
+                | Self::Cuda10
+                | Self::Cuda11
+                | Self::Cuda12
+                | Self::Cuda13
+                | Self::Cuda14
+                | Self::Cuda15
+        )
+    }
+}
+
 #[pyclass]
 #[derive(Clone, Copy, PartialEq, Eq, PartialOrd, Ord)]
 pub struct TorchDeviceDtype {
@@ -1230,6 +1281,10 @@ impl TorchDeviceDtypeOp {
         device: None,
         dtype: None,
     };
+    pub const TENSOR_DEFAULT: Self = TorchDeviceDtypeOp {
+        device: Some(TorchDevice::Cpu),
+        dtype: Some(TorchDtype::float32),
+    };
 
     pub fn combine(self, other: Self) -> Result<Self> {
         let out = Self {
@@ -1257,7 +1312,7 @@ impl TorchDeviceDtypeOp {
 
     pub fn unwrap_or_defaults(self) -> TorchDeviceDtype {
         TorchDeviceDtype {
-            dtype: self.dtype.unwrap_or_else(|| TorchDtype::float32),
+            dtype: self.dtype.unwrap_or_else(|| TorchDtype::float64),
             device: self.device.unwrap_or_else(|| TorchDevice::Cpu),
         }
     }
