@@ -1,22 +1,13 @@
-use circuit_base::{
-    circuit_utils::total_flops, deep_map_unwrap, parsing::Parser, print_html::PrintHtmlOptions,
-    Add, Array, CircuitNode, CircuitRc, Einsum, GeneralFunction, Symbol,
-};
-use circuit_rewrites::{
-    algebraic_rewrite::einsum_nest_path,
-    circuit_optimizer::{
-        optimize_and_evaluate, optimize_circuit, OptimizationContext, OptimizationSettings,
-    },
-    diag_rewrite::{diags_intersection, diags_union},
+use circuit_base::{deep_map_unwrap, parsing::Parser, CircuitNode, CircuitRc};
+use circuit_rewrites::circuit_optimizer::{
+    optimize_circuit, OptimizationContext, OptimizationSettings,
 };
 use mimalloc::MiMalloc;
 use rr_util::{
-    name::{Name, NameInterner},
-    opt_einsum::{get_disconnected_sugraphs, get_int_to_tensor_appearance, EinsumSpec},
+    name::NameInterner,
     rrfs::get_rrfs_dir,
-    sv,
     tensor_util::{TorchDevice, TorchDeviceDtypeOp, TorchDtype},
-    timed, timed_value, tu8v,
+    timed,
 };
 use uuid::Uuid;
 
