@@ -61,7 +61,6 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
         scheduled_execution::scheduled_evaluate,
     };
     use pyo3::{exceptions::PyValueError, types::PyTuple, PyTypeInfo};
-    use rr_util::opt_einsum::optimize_einsum_spec_cached;
 
     // we assume throughout the codebase that usize is 8 bytes, and otherwise error here
     if !core::mem::size_of::<usize>() == 8 {
@@ -86,8 +85,16 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
         m
     )?)?;
     m.add_function(wrap_pyfunction!(circuit_base::generalfunction::pow, m)?)?;
+    m.add_function(wrap_pyfunction!(
+        circuit_base::generalfunction::multinomial,
+        m
+    )?)?;
     circuit_base::generalfunction::register(py, m)?;
     m.add_class::<circuit_base::generalfunction::GeneralFunctionIndexSpec>()?;
+    m.add_class::<circuit_base::generalfunction::GeneralFunctionExplicitIndexSpec>()?;
+    m.add_class::<circuit_base::generalfunction::GeneralFunctionSetDDSpec>()?;
+    m.add_class::<circuit_base::generalfunction::GeneralFunctionPowSpec>()?;
+    m.add_class::<circuit_base::generalfunction::GeneralFunctionMultinomialSpec>()?;
     m.add_class::<circuit_base::generalfunction::GeneralFunctionSpecTester>()?;
     m.add(
         "GeneralFunctionSpecBase",
@@ -183,7 +190,6 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     m.add_function(wrap_pyfunction!(circuit_base::print::oom_fmt_py, m)?)?;
 
     m.add_function(wrap_pyfunction!(circuit_base::check_evaluable, m)?)?;
-    m.add_function(wrap_pyfunction!(circuit_base::evaluate_device_dtype, m)?)?;
 
     m.add_function(wrap_pyfunction!(add_collapse_scalar_inputs, m)?)?;
     m.add_function(wrap_pyfunction!(add_deduplicate, m)?)?;
@@ -214,6 +220,10 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
         circuit_rewrites::generalfunction_rewrite::generalfunction_evaluate_simple,
         m
     )?)?;
+    m.add_function(wrap_pyfunction!(
+        circuit_rewrites::generalfunction_rewrite::generalfunction_gen_index_const_to_index,
+        m
+    )?)?;
     m.add_function(wrap_pyfunction!(concat_pull_removable_axes, m)?)?;
     m.add_function(wrap_pyfunction!(einsum_pull_removable_axes, m)?)?;
     m.add_function(wrap_pyfunction!(add_make_broadcasts_explicit, m)?)?;
@@ -436,6 +446,10 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     m.add_class::<get_update_node::sampler::RandomSampleSpec>()?;
     m.add_class::<get_update_node::sampler::RunDiscreteVarAllSpec>()?;
     m.add_class::<get_update_node::sampler::Sampler>()?;
+    m.add_function(wrap_pyfunction!(
+        get_update_node::sampler::default_hash_seeder,
+        m
+    )?)?;
     m.add_function(wrap_pyfunction!(
         get_update_node::matcher_debug::append_matchers_to_names,
         m
@@ -492,6 +506,10 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
         rr_util::tensor_db::sync_all_unsynced_tensors,
         m
     )?)?;
+    m.add_function(wrap_pyfunction!(
+        rr_util::tensor_db::sync_specific_tensors,
+        m
+    )?)?;
     m.add_function(wrap_pyfunction!(
         rr_util::tensor_db::migrate_tensors_from_old_dir,
         m
